//
//      Copyright (C) by S.Tomabechi 2001
//      All rights reserved.
//
//      represent an odd prime by sum of squares (Euler)
//

#include "Mint.hpp"

// return value :   true sum of squares p=X^2+Y^2
//                  false : is not sum of squares
bool SumSquare(Mint& p,Mint& X,Mint& Y)
{
    Mint x,y ;
    Mint n ;
    Mint alpha,beta ;
    Mint a,b ;
    Mint N ;

    if( (long(p)&3)!=1 ) return false ;

    if( Kronecker(p-1,p)!=1 ) return false ;  // (-1/p)=1 ?

    X=sqrt((p-1),p) ;

    Y=1 ;
    while( X*X+Y*Y>p ) {
        x=amod(X,p) ;
        y=amod(Y,p) ;
        n=x*x+y*y ;
        n/=p ;
        if( n==1  ) {
            break ;
        }
        else {
            a=amod(x,n) ;
            b=amod(y,n) ;
            alpha=(x-a)/n ;
            beta=(y-b)/n ;
            N=(a*a+b*b)/n ;
            X=N+a*alpha+b*beta ;
            if( sign(X)<0 ) ~X ;
            Y=a*beta-b*alpha ;
            if( sign(Y)<0 ) ~Y ;
        }
    }
    return true ;


}


#include <stdlib.h>

int main(void)
{
    unsigned long p ;
    Mint P ;
    Mint x,y ;

    p=12 ;
    for( ; ; ) {
//        cout << "input a number " ;
//        cin >> p ;
        if( p==0 ) break ;
        p=nextprm(p) ;
        P=p ;
        if( SumSquare(P,x,y) ) {
//            cout << endl << x << " " << y ;
            cout << endl << P << " " << x*x << " + " << y*y ;
        }
 //       else cout << endl << "failed " ;
        if( p>1000000) break ;
//	break ;
    }
    return 1 ;

}
