//
//  Copyright (c) by S.Tomabechi 1993,1995,2000 Ver 2.40
//  All rights reserved.
//
//  complex number class
//
//  base field  Fint
//
#ifndef __COMPLEX_HPP
#define __COMPLEX_HPP

#include "Fint.hpp"
#include "MintFunc.hpp"
#include "FintFunc.hpp"
#include "Matrix.hpp"

class PolyC ;

class Complex
{
  friend class PolyC ;
public:
  Fint re,im ;                  // real part and imaginary part

public:
  Complex() {}
  Complex(const Fint& _re) ;
  Complex(const Fint& _re,const Fint& _im) ;
  Complex(const Complex& c) ;
  ~Complex(){}
                                               // operators
  Complex& operator=(const Complex& c) ;
  Complex& operator=(const Fint& s) ;
  Complex& operator=(const Mint& c) ;
  Complex& operator=(const short n) ;
  Complex& operator=(const int n) ;
  Complex& operator=(const long n) ;
  Complex& operator=(const unsigned long n) ;

  Complex& operator+=(const Complex& c) ;
  Complex& operator+=(const Fint& s) ;
  Complex& operator+=(const Mint& n) ;
  Complex& operator+=(const short n) ;
  Complex& operator+=(const int n) ;
  Complex& operator+=(const long n) ;
  Complex& operator+=(const unsigned long n) ;
  Complex& operator+=(const double d) ;
  Complex& operator+=(const long double& d) ;

  Complex operator+(const Complex& b) const ;
  Complex operator+(const Fint& b) const ;
  Complex operator+(const Mint& b) const ;
  Complex operator+(const short b) const ;
  Complex operator+(const int b) const ;
  Complex operator+(const long b) const ;
  Complex operator+(const unsigned long b) const ;
  Complex operator+(const double b) const ;
  Complex operator+(const long double& b) const ;

  Complex& operator-=(const Complex& c) ;
  Complex& operator-=(const Fint& s) ;
  Complex& operator-=(const Mint& n) ;
  Complex& operator-=(const long n) ;
  Complex& operator-=(const unsigned long n) ;

  Complex operator-(const Complex& b) const ;
  Complex operator-(const Fint& b) const ;
  Complex operator-(const Mint& b) const ;
  Complex operator-(const int b) const ;
  Complex operator-(const long b) const ;
  Complex operator-(const unsigned long b) const ;
  Complex operator-(const double& b) const ;
  Complex operator-(const long double& b) const ;

  Complex& operator*=(const Complex& c) ;
  Complex& operator*=(const Fint& s) ;
  Complex& operator*=(const Mint& n) ;
  Complex& operator*=(const short n) ;
  Complex& operator*=(const int n) ;
  Complex& operator*=(const long n) ;
  Complex& operator*=(const unsigned long n) ;
  Complex& operator*=(const double& d) ;
  Complex& operator*=(const long double& d) ;

  Complex operator*(const Complex&  c) const ;
  Complex operator*(const Fint&  c) const ;
  Complex operator*(const Mint&  c) const ;
  Complex operator*(const short  c) const ;
  Complex operator*(const int c) const ;
  Complex operator*(const long c) const ;
  Complex operator*(const unsigned long c) const ;
  Complex operator*(const double& b) const ;
  Complex operator*(const long double& b) const ;

  Complex& operator/=(const Complex& c) ;
  Complex& operator/=(const Fint& s) ;
  Complex& operator/=(const Mint& n) ;
  Complex& operator/=(const short n) ;
  Complex& operator/=(const int n) ;
  Complex& operator/=(const long n) ;
  Complex& operator/=(const unsigned long n) ;

  Complex operator/(const Complex& b) const ;
  Complex operator/(const Fint& b) const ;
  Complex operator/(const Mint& b) const ;
  Complex operator/(const short b) const ;
  Complex operator/(const int b) const ;
  Complex operator/(const long b) const ;
  Complex operator/(const unsigned long b) const ;
  Complex operator/(const double& b) const ;
  Complex operator/(const long double& b) const ;

  Complex  operator+() ;
  Complex  operator-() ;     // negation
  Complex& operator~() ;     // negation

                             // comparison
  bool operator==(const Complex& c) const ;
  bool operator==(const Fint& s) const ;
  bool operator==(const short n) const ;
  bool operator==(const int n) const ;
  bool operator==(const long n) const ;
  bool operator==(const unsigned long n) const ;

  bool operator!=(const Complex& c) const ;
  bool operator!=(const Fint& s) const ;
  bool operator!=(const long n) const ;

  friend Complex operator+(const Fint& s,const Complex& c) ;
  friend Complex operator+(const Mint& n,const Complex& c) ;
  friend Complex operator+(const short a,const Complex& b) ;
  friend Complex operator+(const int a,const Complex& b) ;
  friend Complex operator+(const long n,const Complex& c) ;
  friend Complex operator+(const unsigned long a,const Complex& b) ;
  friend Complex operator+(const double& a,const Complex& b) ;
  friend Complex operator+(const long double& a,const Complex& b) ;


  friend Complex operator-(const Fint& s,const Complex& c) ;
  friend Complex operator-(const Mint& n,const Complex& c) ;
  friend Complex operator-(const short n,const Complex& c) ;
  friend Complex operator-(const int n,const Complex& c) ;
  friend Complex operator-(const long n,const Complex& c) ;
  friend Complex operator-(const unsigned long a,const Complex& b) ;
  friend Complex operator-(const double& a,const Complex& b) ;
  friend Complex operator-(const long double& a,const Complex& b) ;

  friend Complex operator*(const Fint& s,const Complex& c) ;
  friend Complex operator*(const Mint& n,const Complex& c) ;
  friend Complex operator*(const short a,const Complex& b) ;
  friend Complex operator*(const int a,const Complex& b) ;
  friend Complex operator*(const long n,const Complex& c) ;
  friend Complex operator*(const unsigned long a,const Complex& b) ;
  friend Complex operator*(const double& a,const Complex& b) ;
  friend Complex operator*(const long double& a,const Complex& b) ;

  friend Complex operator/(const Fint& a,const Complex& b) ;
  friend Complex operator/(const Mint& a,const Complex& b) ;
  friend Complex operator/(const short a,const Complex& b) ;
  friend Complex operator/(const int a,const Complex& b) ;
  friend Complex operator/(const long a,const Complex& b) ;
  friend Complex operator/(const unsigned long a,const Complex& b) ;
  friend Complex operator/(const double& a,const Complex& b) ;
  friend Complex operator/(const long double& a,const Complex& b) ;

  friend bool operator==(const Fint& s,const Complex& c) ;
  friend bool operator==(const long n,const Complex& c) ;

  friend bool operator!=(const Fint& s,const Complex& c) ;
  friend bool operator!=(const long n,const Complex& c) ;

  friend bool operator>(const Complex& a,const Complex& b) { return true ; }
  friend bool operator<(const Complex& a,const Complex& b) { return true ; }
  friend bool operator>(Complex& c,long n) { return true ; }
  friend bool operator<(Complex& c,long n) { return true ; }
  friend bool operator>(long n,Complex& c) { return true ; }
  friend bool operator<(long n,Complex& c) { return true ; }

  friend const Fint& real(const Complex& c) ;
  friend const Fint& imag(const Complex& c) ;
  friend Fint& real(Complex& c) ;
  friend Fint& imag(Complex& c) ;
  friend Complex conj(const Complex& c) ;
  friend Fint  abs(const Complex& c) ;
  friend Fint  norm(const Complex& c) ;
  friend Complex power(const Complex& x,const Complex& y) ;
  friend Complex exp(const Complex& x) ;
  friend Complex log(const Complex& x) ;
  friend Complex sqrt(const Complex& x) ;
  friend Complex cos(const Complex& x) ;
  friend Complex sin(const Complex& x) ;
  friend Complex tan(const Complex& x) ;
  friend Complex atan(const Complex& x) ;
  friend Complex cosh(const Complex& x) ;
  friend Complex sinh(const Complex& x) ;
  friend Complex power(const Complex& x,const Mint& n) ;
  friend Complex power(const Complex& x,const Fint& y) ;
  friend Complex power(const Fint& x,const Complex& y) ;
  friend Complex power(const Mint& n,const Complex& c) ;
  friend Complex power(const Complex& x,unsigned long n) ;
  friend Complex power(const Complex& x,long n) ;
  friend Complex power(const Complex& x,int n) ;
  friend void Swap(Complex& a,Complex& b) ;

  friend ostream& operator<<(ostream& os,const Complex& c) ;
  friend istream& operator>>(istream& is,Complex& c) ;
  friend void write(ostream& os,const Complex& c) ;
  friend void read(istream& is,Complex& c) ;

public:

  Complex& isAdd(const Complex& a,const Complex& b) ;
  Complex& isAdd(const Complex& a,const Fint& b) ;
  Complex& isAdd(const Complex& a,const Mint& b) ;
  Complex& isAdd(const Complex& a,const short b) ;
  Complex& isAdd(const Complex& a,const int b) ;
  Complex& isAdd(const Complex& a,const long b) ;
  Complex& isAddUL(const Complex& a,const unsigned long b) ;
  Complex& isAdd(const Complex& a,const double& b) ;
  Complex& isAdd(const Complex& a,const long double& b) ;

  Complex& isSub(const Complex& a,const Complex& b) ;
  Complex& isSub(const Complex& a,const Fint& b) ;
  Complex& isSub(const Complex& a,const Mint& b) ;
  Complex& isSub(const Complex& a,const short b) ;
  Complex& isSub(const Complex& a,const int b) ;
  Complex& isSub(const Complex& a,const long b) ;
  Complex& isSubUL(const Complex& a,const unsigned long b) ;
  Complex& isSub(const Complex& a,const double& b) ;
  Complex& isSub(const Complex& a,const long double& b) ;

  Complex& isSub(const Fint& a,const Complex& b) ;
  Complex& isSub(const Mint& a,const Complex& b) ;
  Complex& isSub(const short a,const Complex& b) ;
  Complex& isSub(const int a,const Complex& b) ;
  Complex& isSub(const long a,const Complex& b) ;
  Complex& isSub(const unsigned long,const Complex& b) ;
  Complex& isSub(const double& a,const Complex& b) ;
  Complex& isSub(const long double& a,const Complex& b) ;


  Complex& isMul(const Complex& a,const Complex& b) ;
  Complex& isMul(const Complex& a,const Fint& b) ;
  Complex& isMul(const Complex& a,const Mint& b) ;
  Complex& isMul(const Complex& a,const short b) ;
  Complex& isMul(const Complex& a,const int b) ;
  Complex& isMul(const Complex& a,const long b) ;
  Complex& isMulUL(const Complex& a,const unsigned long b) ;
  Complex& isMul(const Complex& a,const double& b) ;
  Complex& isMul(const Complex& a,const long double& b) ;

  Complex& isDiv(const Complex& a,const Complex& b) ;
  Complex& isDiv(const Complex& a,const Fint& b) ;
  Complex& isDiv(const Complex& a,const Mint& b) ;
  Complex& isDiv(const Complex& a,const short b) ;
  Complex& isDiv(const Complex& a,const int b) ;
  Complex& isDiv(const Complex& a,const long b) ;
  Complex& isDivUL(const Complex& a,const unsigned long b) ;
  Complex& isDiv(const Complex& a,const double& b);
  Complex& isDiv(const Complex& a,const long double& b) ;

  Complex& isDivF(const Mint& a,const Complex& b) ;
  Complex& isDiv(const Fint& a,const Complex& b) ;
  Complex& isDiv(const short a,const Complex& b) ;
  Complex& isDiv(const int a,const Complex& b) ;
  Complex& isDiv(const long a,const Complex& b) ;
  Complex& isDiv(const unsigned long a,const Complex& b) ;
  Complex& isDiv(const double& a,const Complex& b) ;
  Complex& isDiv(const long double& a,const Complex& b) ;

public:
  static Complex _Ctemp ;

} ;


inline Complex& Complex::isAdd(const Complex& a,const Complex& b)
{
  re.isAdd(a.re,b.re) ;
  im.isAdd(a.im,b.im) ;
  return *this ;
}

inline Complex& Complex::isAdd(const Complex& a,const Fint& b)
{
  re.isAdd(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isAdd(const Complex& a,const Mint& b)
{
  re.isAdd(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isAdd(const Complex& a,const short b)
{
  return isAdd(a,long(b)) ;
}

inline Complex& Complex::isAdd(const Complex& a,const int b)
{
  return isAdd(a,long(b)) ;
}

inline Complex& Complex::isAdd(const Complex& a,const long b)
{
  re.isAdd(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isAddUL(const Complex& a,const unsigned long b)
{
  re.isAddUL(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isAdd(const Complex& a,const double& b)
{
  return isAdd(a,(long double)b) ;

}

inline Complex& Complex::isAdd(const Complex& a,const long double& b)
{
  re.isAdd(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSub(const Complex& a,const Complex& b)
{
  re.isSub(a.re,b.re) ;
  im.isSub(a.im,b.im) ;
  return *this ;
}

inline Complex& Complex::isSub(const Complex& a,const Fint& b)
{
  re.isSub(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSub(const Complex& a,const Mint& b)
{
  re.isSub(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSub(const Complex& a,const short b)
{
  return isSub(a,long(b)) ;
}

inline Complex& Complex::isSub(const Complex& a,const int b)
{
  return isSub(a,long(b)) ;
}

inline Complex& Complex::isSub(const Complex& a,const long b)
{
  re.isSub(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSubUL(const Complex& a,const unsigned long b)
{
  re.isSubUL(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSub(const Complex& a,const double& b)
{
  return isSub(a,(long double)b) ;
}

inline Complex& Complex::isSub(const Complex& a,const long double& b)
{
  re.isSub(a.re,b) ;
  im=a.im ;
  return *this ;
}

inline Complex& Complex::isSub(const Fint& a,const Complex& b)
{
  re.isSub(a,b.re) ;
  im.isNeg(b.im) ;
  return *this ;
}


inline Complex& Complex::isSub(const Mint& a,const Complex& b)
{
  re.isSub(a,b.re) ;
  im.isNeg(b.im) ;
  return *this ;
}

inline Complex& Complex::isSub(const short a,const Complex& b)
{
  return isSub(long(a),b) ;
}

inline Complex& Complex::isSub(const int a,const Complex& b)
{
  return isSub(long(a),b) ;
}

inline Complex& Complex::isSub(const long a,const Complex& b)
{
  re.isSub(a,b.re) ;
  im.isNeg(b.im) ;
  return *this ;
}


inline Complex& Complex::isSub(const unsigned long a,const Complex& b)
{
  ~re.isSubUL(b.re,a) ;     // re = -(b.re-a) ;
  im.isNeg(b.im) ;
  return *this ;
}

inline Complex& Complex::isSub(const double& a,const Complex& b)
{
  return isSub((long double)a,b) ;
}

inline Complex& Complex::isSub(const long double& a,const Complex& b)
{
  re.isSub(a,b.re) ;
  im.isNeg(b.im) ;
  return *this ;
}

inline Complex& Complex::isMul(const Complex& a,const Fint& b)
{
  re.isMul(a.re,b) ;
  im.isMul(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isMul(const Complex& a,const Mint& b)
{
  re.isMul(a.re,b) ;
  im.isMul(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isMul(const Complex& a,const short b)
{
  return isMul(a,long(b)) ;
}

inline Complex& Complex::isMul(const Complex& a,const int b)
{
  return isMul(a,long(b)) ;
}

inline Complex& Complex::isMul(const Complex& a,const long b)
{
  re.isMul(a.re,b) ;
  im.isMul(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isMulUL(const Complex& a,const unsigned long b)
{
  re.isMulUL(a.re,b) ;
  im.isMulUL(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isMul(const Complex& a,const double& b)
{
  return isMul(a,(long double)b) ;
}

inline Complex& Complex::isMul(const Complex& a,const long double& b)
{
  re.isMul(a.re,b) ;
  im.isMul(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDiv(const Complex& a,const Fint& b)
{
  re.isDivF(a.re,b) ;
  im.isDivF(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDiv(const Complex& a,const Mint& b)
{
  re.isDiv(a.re,b) ;
  im.isDiv(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDiv(const Complex& a,const short b)
{
  return isDiv(a,long(b)) ;
}

inline Complex& Complex::isDiv(const Complex& a,const int b)
{
  return isDiv(a,long(b)) ;
}

inline Complex& Complex::isDiv(const Complex& a,const long b)
{
  re.isDiv(a.re,b) ;
  im.isDiv(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDivUL(const Complex& a,const unsigned long b)
{
  re.isDivUL(a.re,b) ;
  im.isDivUL(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDiv(const Complex& a,const double& b)
{
  return isDiv(a,(long double)b) ;
}

inline Complex& Complex::isDiv(const Complex& a,const long double& b)
{
  isDiv(a.re,b) ;
  isDiv(a.im,b) ;
  return *this ;
}

inline Complex& Complex::isDiv(const Fint& a,const Complex& b)
{
  return isDivF((Mint&)a,b) ;
}
inline Complex& Complex::isDiv(const short a,const Complex& b)
{
  return isDiv(long(a),b) ;
}

inline Complex& Complex::isDiv(const int a,const Complex& b)
{
  return isDiv(long(a),b) ;
}

inline Complex& Complex::isDiv(const long a,const Complex& b)
{
  return isDivF(Mint::_Mtemp=a,b) ;
}


inline Complex& Complex::isDiv(const unsigned long a,const Complex& b)
{
  return isDivF(Mint::_Mtemp=a,b) ;
}

inline Complex& Complex::isDiv(const double& a,const Complex& b)
{
  return isDiv((long double)a,b) ;
}

inline Complex& Complex::isDiv(const long double& a,const Complex& b)
{
  Fint::_uFtemp.fDigit=0 ;
  return isDivF((Mint&)(Fint::_uFtemp=a),b) ;
}


inline Complex& Complex::operator~()
{
  ~re ;
  ~im ;
  return *this ;
}


///////////////////////////////////////////////////////
//
//  constructors
//
inline Complex::Complex(const Fint& _re)            // construct from real numbner
{
  re=_re ;
  im=0 ;
}

inline Complex::Complex(const Fint& _re,const Fint& _im)    // construct from real part and imaginary part
                                                            //¿ʬ,ʬ)
{
  re=_re ;
  im=_im ;
}

inline Complex::Complex(const Complex& c)    // copy constructor
{
  re=c.re ;
  im=c.im ;
}

///////////////////////////////////////////
//
//  assignment
//
inline Complex& Complex::operator=(const Complex& c)
{
  re=c.re ;
  im=c.im ;
  return *this ;
}

inline Complex& Complex::operator=(const Fint& s)
{
  re=s ; im=0 ;
  return *this ;
}

inline Complex& Complex::operator=(const Mint& s)
{
  re=s ; im=0 ;
  return *this ;
}

inline Complex& Complex::operator=(const short n)
{
  return operator=(long(n)) ;
}

inline Complex& Complex::operator=(const int n)
{
  return operator=(long(n)) ;
}

inline Complex& Complex::operator=(const long n)
{
  re=n ;
  im=0 ;
  return *this ;
}

inline Complex& Complex::operator=(const unsigned long n)
{
  re=n ;
  im=0 ;
  return *this ;
}


////////////////////////////////
//
//  addition
//
inline Complex& Complex::operator+=(const Complex& c)
{
  return isAdd(*this,c) ;
}

inline Complex& Complex::operator+=(const Fint& s)
{
  return isAdd(*this,s) ;
}

inline Complex& Complex::operator+=(const Mint& n)
{
  return isAdd(*this,n) ;
}


inline Complex& Complex::operator+=(const short n)
{
  return isAdd(*this,n) ;
}


inline Complex& Complex::operator+=(const int n)
{
  return isAdd(*this,n) ;
}


inline Complex& Complex::operator+=(const long n)
{
  return isAdd(*this,n) ;
}

inline Complex& Complex::operator+=(const unsigned long n)
{
  return isAddUL(*this,n) ;
}


inline Complex& Complex::operator+=(const double d)
{
  return isAdd(*this,d) ;
}

inline Complex& Complex::operator+=(const long double& d)
{
  return isAdd(*this,d) ;
}


inline Complex Complex::operator+(const Complex& b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const Fint& b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const Mint& b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const short b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const int b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const unsigned long b) const
{
  return Complex::_Ctemp.isAddUL(*this,b) ;
}

inline Complex Complex::operator+(const long b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex Complex::operator+(const double b) const
{
  return Complex::_Ctemp.isAdd(*this,b) ;
}

inline Complex operator+(const Fint& a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}



inline Complex operator+(const Mint& a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}

inline Complex operator+(const short a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}

inline Complex operator+(const int a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}

inline Complex operator+(const long a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}

inline Complex operator+(const unsigned long a,const Complex& b)
{
  return Complex::_Ctemp.isAddUL(b,a) ;
}

inline Complex operator+(const double& a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}

inline Complex operator+(const long double& a,const Complex& b)
{
  return Complex::_Ctemp.isAdd(b,a) ;
}


/////////////////////////////////
//
//  subtraction
//
inline Complex& Complex::operator-=(const Complex& c)
{
  return isSub(*this,c) ;
}

inline Complex& Complex::operator-=(const Fint& s)
{
  return isSub(*this,s) ;
}

inline Complex& Complex::operator-=(const Mint& n)
{
  return isSub(*this,n) ;
}
inline Complex& Complex::operator-=(const long n)
{
  return isSub(*this,n) ;
}

inline Complex& Complex::operator-=(const unsigned long n)
{
  return isSubUL(*this,n) ;
}

inline Complex Complex::operator-(const Complex& b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const Fint& b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const Mint& b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const int b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const long b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const unsigned long b) const
{
  return Complex::_Ctemp.isSubUL(*this,b) ;
}

inline Complex Complex::operator-(const double& b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex Complex::operator-(const long double& b) const
{
  return Complex::_Ctemp.isSub(*this,b) ;
}

inline Complex operator-(const Fint& a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const Mint& a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const short a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const int a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const long a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const unsigned long a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const double& a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}

inline Complex operator-(const long double& a,const Complex& b)
{
  return Complex::_Ctemp.isSub(a,b) ;
}


/////////////////////////////////////
//
// multiplication
//
inline Complex& Complex::operator*=(const Complex& c)
{
  return isMul(*this,c) ;
}

inline Complex& Complex::operator*=(const Fint& s)
{
  return isMul(*this,s) ;
}

inline Complex& Complex::operator*=(const Mint& n)
{
  return isMul(*this,n) ;
}

inline Complex& Complex::operator*=(const short n)
{
  return isMul(*this,n) ;
}

inline Complex& Complex::operator*=(const int n)
{
  return isMul(*this,n) ;
}

inline Complex& Complex::operator*=(const long n)
{
  return isMul(*this,n) ;
}


inline Complex& Complex::operator*=(const unsigned long n)
{
  return isMulUL(*this,n) ;
}

inline Complex Complex::operator*(const Complex& b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}

inline Complex Complex::operator*(const Fint& b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}

inline Complex Complex::operator*(const Mint& b) const
{
  return Complex::_Ctemp.isMul(*this,b);
}

inline Complex Complex::operator*(const short b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}

inline Complex Complex::operator*(const int b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}

inline Complex Complex::operator*(const long b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}

inline Complex Complex::operator*(const unsigned long b) const
{
  return Complex::_Ctemp.isMulUL(*this,b) ;
}

inline Complex Complex::operator*(const double& b) const
{
  return Complex::_Ctemp.isMul(*this,(long double)b) ;
}

inline Complex Complex::operator*(const long double& b) const
{
  return Complex::_Ctemp.isMul(*this,b) ;
}











inline Complex operator*(const Fint& a,const Complex& b )
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const Mint& a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const short a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const int a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const long a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const unsigned long a,const Complex& b)
{
  return Complex::_Ctemp.isMulUL(b,a) ;
}

inline Complex operator*(const double& a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}

inline Complex operator*(const long double& a,const Complex& b)
{
  return Complex::_Ctemp.isMul(b,a) ;
}


/////////////////////////////////////////
//
//  division
//
inline Complex Complex::operator/(const Complex& b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}

inline Complex Complex::operator/(const Fint& b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}

inline Complex Complex::operator/(const Mint& b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}


inline Complex Complex::operator/(const short b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}


inline Complex Complex::operator/(const int b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}


inline Complex Complex::operator/(const long b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}

inline Complex Complex::operator/(const unsigned long b) const
{
  return Complex::_Ctemp.isDivUL(*this,b) ;
}

inline Complex Complex::operator/(const double& b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}

inline Complex Complex::operator/(const long double& b) const
{
  return Complex::_Ctemp.isDiv(*this,b) ;
}


inline Complex operator/(const Fint& a,const Complex& b)
{
  return Complex::_Ctemp.isDivF((Mint&)a,b) ;
}

inline Complex operator/(const Mint& a,const Complex& b)
{
  return Complex::_Ctemp.isDivF(a,b) ;
}

inline Complex operator/(const short a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}

inline Complex operator/(const int a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}
inline Complex operator/(const long a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}

inline Complex operator/(const unsigned long a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}

inline Complex operator/(const double& a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}

inline Complex operator/(const long double& a,const Complex& b)
{
  return Complex::_Ctemp.isDiv(a,b) ;
}


inline Complex& Complex::operator/=(const Complex& c)
{
  return  isDiv(*this,c) ;
}


inline Complex& Complex::operator/=(const Fint& s)
{
  return  isDiv(*this,s) ;
}

inline Complex& Complex::operator/=(const Mint& n)
{
  return isDiv(*this,n) ;
}

inline Complex& Complex::operator/=(const short n)
{
  return isDiv(*this,n) ;
}

inline Complex& Complex::operator/=(const int n)
{
  return isDiv(*this,n) ;
}

inline Complex& Complex::operator/=(const long b)
{
  return isDiv(*this,b) ;
}

inline Complex& Complex::operator/=(const unsigned long n)
{
  return isDivUL(*this,n) ;
}

inline Complex Complex::operator+()
{
  return *this ;
}

inline Complex Complex::operator-()
{
  _Ctemp.re.isNeg(re) ;
  _Ctemp.im.isNeg(im) ;
  return _Ctemp ;
}

inline bool Complex::operator==(const Complex& c) const
{
  return re==c.re && im==c.im ;
}

inline bool Complex::operator==(const Fint& s) const
{
  return re==s && im==0 ;
}

inline bool Complex::operator==(const short n) const
{
  return re==n && im==0 ;
}

inline bool Complex::operator==(const int n) const
{
  return re==n && im==0 ;
}

inline bool Complex::operator==(const long n) const
{
  return re==n && im==0 ;
}

inline bool Complex::operator==(const unsigned long n) const
{
  return re==n && im==0 ;
}

inline bool operator==(const long n,const Complex& c)
{
  return c.operator==(n) ;
}

inline bool Complex::operator!=(const Complex& c) const
{
  return re!=c.re || im!=c.im ;
}

inline bool Complex::operator!=(const Fint& s) const
{
  return im!=0 || re!=s ;
}

inline bool Complex::operator!=(const long n) const
{
  return im!=0 || re!=n ;
}

inline bool operator!=(const long n,const Complex& c)
{
  return c.operator!=(n) ;
}

inline bool operator==(const Fint& s,const Complex& c)
{
  return c==s ;
}

inline bool operator!=(const Fint& s,const Complex& c)
{
  return c.operator!=(s) ;
}

inline Complex conj(const Complex& c)        // conjugate
                                             // 
{
  Complex::_Ctemp=c ;

  ~(Complex::_Ctemp.im) ;
  return Complex::_Ctemp ;
}


inline const Fint& imag(const Complex& c)     // imaginary part
                                              // ʬ
{
  return c.im ;
}

inline Fint& real(Complex& c)     // ¿ʬ. ͤǤ뤳Ȥ
{
  return c.re ;
}

inline const Fint& real(const Complex& c)  // ¿ʬ. ͤǤ뤳Ȥ
{
  return c.re ;
}


inline Fint& imag(Complex& c)     // ʬ
{
  return c.im ;
}

inline void Swap(Complex& a,Complex& b)
{
  Swap(a.re,b.re) ;
  Swap(a.im,b.im) ;
}

inline ostream& operator<<(ostream& os,const Complex& c)
{
  return os << "(" << real(c) << "," << imag(c) << ")" ;
}

inline istream& operator>>(istream& is,Complex& c)
{
  return is >> c.re >> c.im ;
}

inline void write(ostream& os,const Complex& c)
{
  write(os,c.re) ;
  write(os,c.im) ;
}

inline void read(istream& is,Complex& c)
{
  read(is,c.re) ;
  read(is,c.im) ;
}

inline Complex power(const Complex& x,int n)
{
  return power(x,long(n)) ;
}

// theta function
// \sum (2zn+n\tau\n)
Complex theta(const Complex& zz,const Complex& tt) ;

Complex dctheta(const Complex& z,const Complex& t,
                const unsigned long n,const unsigned long a,
                const unsigned long b) ;
Complex ctheta(const Complex& z,const Complex& t,
                const unsigned long n,const unsigned long a,
                const unsigned long b) ;

// t00  zero
// z=(p+1/2)+(q+1/2)
inline Complex t00(const Complex& z,const Complex& t)
{
    return ctheta(z,t,2,0,0) ;
}
inline Complex t01(const Complex& z,const Complex& t)
{
    return ctheta(z,t,2,0,1) ;
}
inline Complex t10(const Complex& z,const Complex& t)
{
    return ctheta(z,t,2,1,0) ;
}

//ȡ
inline Complex t11(const Complex& z,const Complex& t)
{
    return ctheta(z,t,2,1,1) ;
}

// Ϳ줿 t ΰذܤ
//  t ϡΰͤ A Ȥʤͤѹ
void reduction(Complex& t,Matrix<Mint>& A) ;


#endif
