//
//	Copyright (C) by S.Tomabechi 1993,1995,1998,1999
//	All rights reserved.
//
//	finite field with prime characteristic	Fp=Z/pZ
//
//	1998/06/19 const  ɲ
//			   ؿΰ®
//
//	1998/09/03 ɸ 2~31-1 ʲ 2^32-1 ʲѹ
//			   ɸ long Ǥʤ unsigned long ɽ褦ˤ
//
//

#ifndef __FP_HPP
#define __FP_HPP

#include <iostream.h>
#include "IntFuncs.hpp"


class PolyFp ;
class Fp
{
  friend class PolyFp ;					// polynomial with coefficient in finite field
private:
public:
  static unsigned long ch ;				// characteristic
  unsigned long Zp ;					// value
public:
  Fp():Zp(0L){}							// do nothing

  Fp(const int n) ;
  Fp(const long n) ;					// construct from long integer
  Fp(const unsigned long n) ;			// construct from unsigned long

  Fp(const Fp& a) ;						// copy constructor

//#ifdef USE_MINT
//	Fp(const Mint& a) { Zp=mod(a,ch) ; }
//#endif

  ~Fp(){}								// destructor do nothing

  Fp& operator=(const Fp& a);			// assignment operator
  Fp& operator=(const long a) ;
  Fp& operator=(const int n) ;
  Fp& operator=(const unsigned long a) ;

										// operators
  Fp& operator+=(const Fp& n) ;
  Fp& operator+=(const long a) ;
  Fp& operator+=(const int n) ;
  Fp& operator+=(const unsigned long n) ;

  Fp  operator+(const Fp& n) const ;
  Fp  operator+(const long n) const ;
  Fp  operator+(const int n) const ;
  Fp  operator+(const unsigned long a) const ;


  Fp& operator-=(const Fp& n) ;
  Fp& operator-=(const long n) ;
  Fp& operator-=(const int n) ;
  Fp& operator-=(const unsigned long n) ;

  Fp  operator-(const Fp& n) const ;
  Fp  operator-(const long n) const ;
  Fp  operator-(const int n) const ;
  Fp  operator-(const unsigned long n) const ;


  Fp& operator*=(const Fp& a) ;
  Fp& operator*=(const long a) ;
  Fp& operator*=(const int a)  ;
  Fp& operator*=(const unsigned long a) ;

  Fp  operator*(const Fp& a) const ;
  Fp  operator*(const long a) const ;
  Fp  operator*(const int a) const ;
  Fp  operator*(const unsigned long a) const ;

  Fp& operator/=(const Fp& a) ;
  Fp& operator/=(const long a) ;
  Fp& operator/=(const int a) ;
  Fp& operator/=(const unsigned long a) ;

  Fp  operator/(const Fp& a) const ;
  Fp  operator/(const long a) const ;
  Fp  operator/(const int n) const ;
  Fp  operator/(const unsigned	long n) const ;

  Fp  operator-() const ;

										// relations
  bool operator==(const Fp& a) const ;
  bool operator==(const long a) const ;
  bool operator==(const int a ) const ;
  bool operator==(const unsigned long a) const ;

  bool operator!=(const Fp& a) const ;
  bool operator!=(const long a) const ;
  bool operator!=(const int a) const ;
  bool operator!=(const unsigned long a) const ;

  bool operator>(const Fp& a) const ;  // absurd comparison
  bool operator>(const long n) const ;
  bool operator>(const int n) const ;
  bool operator>(const unsigned long n) const ;

  bool operator<(const Fp& a) const ;
  bool operator<(const long n) const ;
  bool operator<(const unsigned long n) const ;
  bool operator<(const int n) const ;
	   operator unsigned long() const ;		  // conversion operator  lift to integer ring Z

//	bool isZero(void) const ;

  friend unsigned long SetChar(unsigned long _char) ;	  // set characteristic
  friend unsigned long Char(void) ;						  // get current characteristic
  friend Fp operator+(const long a,const Fp& x) ;
  friend Fp operator+(const int a,const Fp& x) ;
  friend Fp operator+(const unsigned long a,const Fp& x) ;
  friend Fp operator-(const long a,const Fp& x) ;
  friend Fp operator-(const int a,const Fp& x) ;
  friend Fp operator-(const unsigned long a,const Fp& x) ;
  friend Fp operator*(const long a,const Fp& x) ;
  friend Fp operator*(const int a,const Fp& x) ;
  friend Fp operator*(const unsigned long a,const Fp& x) ;
  friend Fp operator/(const long a,const Fp& x) ;
  friend Fp operator/(const int a,const Fp& x) ;
  friend Fp operator/(const unsigned long a,const Fp& x) ;
  friend Fp power(const Fp& a,const unsigned long n) ;
  friend ::ostream& operator<<(::ostream& os,const Fp& a) ;
  friend ::istream& operator>>(::istream& is,const Fp& a) ;
protected:
public:
  static Fp _Fptemp ;
} ;

inline Fp::Fp(const int n)
{
  Zp=mod(long(n),ch) ;
}

inline Fp::Fp(const long n)		//	construct form long integer
{
  Zp=mod(n,ch) ;
}

inline Fp::Fp(const unsigned long n)	 // construct from	unsigned long integer
{
  Zp=n%ch ;
}

inline Fp::Fp(const Fp& n)	 // copy constructor
{
  Zp=n.Zp ;
}

inline Fp& Fp::operator=(const Fp& n)	  // assign
{
  Zp=n.Zp ;
  return *this ;
}

inline Fp& Fp::operator=(const long n)	  // assign	 n mod p
{
  Zp=mod(n,ch) ;
  return *this ;
}

inline Fp& Fp::operator=(const int n)
{
  return operator=(long(n)) ;
}

inline Fp& Fp::operator=(const unsigned long n)	  // assign n mod p
{
  Zp=n%ch ;
  return *this ;
}

inline Fp Fp::operator+(const Fp& n) const	   // addition
{
  _Fptemp.Zp=isAdd(Zp,n.Zp,ch) ;
  return _Fptemp ;
}

inline Fp& Fp::operator+=(const Fp& n)
{
  Zp=isAdd(Zp,n.Zp,ch) ;
  return *this ;
}

inline Fp Fp::operator+(const long n) const
{
  _Fptemp.Zp=isAdd(Zp,mod(n,ch),ch) ;
  return _Fptemp ;
}

inline Fp Fp::operator+(const int n) const
{
  return operator+(long(n)) ;
}

inline Fp Fp::operator+(const unsigned long n) const
{
  _Fptemp.Zp=isAdd(Zp,n,ch) ;
  return _Fptemp ;
}

inline Fp& Fp::operator+=(const long n)
{
  Zp=isAdd(Zp,mod(n,ch),ch) ;
  return *this ;
}

inline Fp& Fp::operator+=(const int	 n)
{
  return operator+=(long(n)) ;
}

inline Fp& Fp::operator+=(const unsigned long n)
{
  Zp=isAdd(Zp,n,ch) ;
  return *this ;
}

inline Fp Fp::operator-(const Fp& n) const		  // subtraction
{
  _Fptemp.Zp=isSub(Zp,n.Zp,ch) ;
  return _Fptemp ;
}

inline Fp& Fp::operator-=(const Fp& n)
{
  Zp=isSub(Zp,n.Zp,ch) ;
  return *this ;
}

inline Fp Fp::operator-(const long n) const
{
  _Fptemp.Zp=isSub(Zp,mod(n,ch),ch) ;
  return _Fptemp ;
}

inline Fp Fp::operator-(const int n) const
{
  return operator-(long(n)) ;
}


inline Fp Fp::operator-(const unsigned long n) const
{
  _Fptemp.Zp=isSub(Zp,n,ch) ;
  return _Fptemp ;
}

inline Fp& Fp::operator-=(const long n)
{
  Zp=isSub(Zp,mod(n,ch),ch) ;
  return *this ;
}

inline Fp& Fp::operator-=(const int n)
{
  return operator-=(long(n)) ;
}

inline Fp& Fp::operator-=(const unsigned long n)
{
  Zp=isSub(Zp,n,ch) ;
  return *this ;
}

inline Fp Fp::operator*(const Fp& n) const		// multiplication
{
  _Fptemp.Zp=mod(Zp,n.Zp,ch) ;
  return _Fptemp ;
}

inline Fp Fp::operator*(const int n) const
{
  return operator*(long(n)) ;
}


inline Fp& Fp::operator*=(const Fp& n)
{
  Zp=mod(Zp,n.Zp,ch) ;
  return *this ;
}

inline Fp Fp::operator*(const long n) const
{
  _Fptemp.Zp=mod(Zp,mod(n,ch),ch) ;
  return _Fptemp ;
}

inline Fp Fp::operator*(const unsigned long n) const
{
  _Fptemp.Zp=mod(Zp,n,ch) ;
  return _Fptemp ;
}

inline Fp operator*(const long n,const Fp& c)
{
  return c*n ;
}

inline Fp operator*(const int n,const Fp& c)
{
  return operator*(long(n),c) ;
}

inline Fp operator*(const unsigned long n,const Fp& c)
{
  return c*n ;
}


inline Fp& Fp::operator*=(const long n)
{
  Zp=mod(Zp,mod(n,ch),ch)  ;
  return *this ;
}

inline Fp& Fp::operator*=(const unsigned long n)
{
  Zp=mod(Zp,n,ch)  ;
  return *this ;
}

inline Fp& Fp::operator*=(const int a)
{
  return operator*=(long(a)) ;
}

inline Fp Fp::operator/(const Fp& n) const		 // division
{
  _Fptemp.Zp=mod(Zp,invmodp(n.Zp,ch),ch) ;
  return _Fptemp ;
}


inline Fp& Fp::operator/=(const Fp& n)
{
  Zp=mod(Zp,invmodp(n.Zp,ch),ch) ;
  return *this ;
}

inline Fp Fp::operator/(const long n) const
{
  _Fptemp.Zp=mod(Zp,invmodp(mod(n,ch),ch),ch) ;
  return _Fptemp ;
}

inline Fp Fp::operator/(const int n) const
{
   return operator/(long(n)) ;
}

inline Fp Fp::operator/(const unsigned long n) const
{
  _Fptemp.Zp=mod(Zp,invmodp(n,ch),ch) ;		   // * n^-1 mod ch
  return _Fptemp ;
}

inline Fp& Fp::operator/=(const long n)
{
  Zp=mod(Zp,invmodp(mod(n,ch),ch),ch) ;
  return *this ;
}

inline Fp& Fp::operator/=(const int n)
{
  return operator/=(long(n)) ;
}

inline Fp& Fp::operator/=(const unsigned long n)
{
  Zp=mod(Zp,invmodp(n,ch),ch) ;
  return *this ;
}

inline Fp operator+(const long n,const Fp& c)
{
  return c+n ;
}

inline Fp operator+(const int n,const Fp& c)
{
  return c.operator+(long(n)) ;
}

inline Fp operator+(const unsigned long n,const Fp& c)
{
  return c+n ;
}

inline Fp operator-(const long n,const Fp& c)
{
  Fp::_Fptemp.Zp=isSub(mod(n,Char()),c.Zp,Char()) ;
  return Fp::_Fptemp ;
}

inline Fp operator-(const int n,const Fp& c)
{
  return operator-(long(n),c) ;
}

inline Fp operator-(const unsigned long n,const Fp& c)
{
  Fp::_Fptemp.Zp=isSub(n,c.Zp,Char()) ;
  return Fp::_Fptemp ;
}


inline Fp Fp::operator-() const
{
  Fp::_Fptemp.Zp=(Zp==0 ? 0:(ch-Zp)) ;
  return Fp::_Fptemp ;
}

inline bool Fp::operator==(const Fp& a) const
{
  return Zp==a.Zp ;
}

inline bool Fp::operator==(const long a) const
{
  return mod(a,ch)==Zp ;
}

inline bool Fp::operator==(const unsigned long a) const
{
  return Zp==(a%ch) ;
}

inline bool Fp::operator==(const int a) const
{
  return operator==(long(a)) ;
}

inline bool Fp::operator!=(const Fp& a) const
{
  return !operator==(Fp(a)) ;
}

inline bool Fp::operator!=(const long a) const
{
  return !operator==(a) ;
}

inline bool Fp::operator!=(const int a) const
{
  return operator!=(long(a)) ;
}


// non-sense operators
inline bool Fp::operator<(const Fp& a) const
{
  return Zp<a.Zp ;
}

inline bool Fp::operator<(const long n) const
{
  return Zp<mod(n,ch) ;
}

inline bool Fp::operator<(const int n) const
{
  return operator<(long(n)) ;
//	return Zp<long(n) ;
}

inline bool Fp::operator>(const Fp& a) const
{
  return Zp>a.Zp ;
}

inline bool Fp::operator>(const long n) const
{
  return Zp>n ;
}

inline bool Fp::operator>(const int n) const
{
  return Zp>long(n) ;
}

inline Fp operator/(const long a,const Fp& x)
{
  Fp::_Fptemp.Zp=mod(mod(a,Fp::ch),invmodp(x.Zp,Fp::ch),Fp::ch) ;
  return Fp::_Fptemp ;
}

inline Fp operator/(const int a,const Fp& x)
{
  return operator/(long(a),x) ;
}

inline unsigned long SetChar(const unsigned long _char)
{
  unsigned long p=Fp::ch ;
  Fp::ch=_char ;
  return p ;
}

inline unsigned long Char(void)				 // return characteristic
{
  return Fp::ch ;
}

inline Fp::operator unsigned long()	 const // ĤؤΥե
{
  return Zp ;
}


inline Fp power(const Fp& a,const unsigned long n)
{
  return Fp(power((unsigned long)a,n,Char())) ;
}


inline ::ostream& operator<<(::ostream& os,const Fp& a)
{
  return os << a.Zp ;
}

inline ::istream& operator>>(::istream& is,const Fp& a)
{
  return is >> a.Zp ;
}

#endif
