//
//  PPMPQS Ver 2.8
//
//  Copyright (C) by S.Tomabechi 1995,1996,1997,1999,2000
//  All rights reserved.
//
//   factorization by PPMPQS
//
//      MPQS ( Multiple Polynomial Quadratic Sieve Method )
//
//      small divisors must be removed  before using PPMPQS
//
//  1995/06/26,1995/11/03
//    modified the function sqrt which find a square root modulo p
//
//  1996/09/18  modified to 32 bit version
//
//  1996/11/25  modified to PPMPQS from PMPQS
//
//  1996/12/20  implement intelligent Gaussian method
//
//  1997/02/15  delete duplicate data
//
//              sieve power of small primes
//
//  1997/03/15 դ뤦ݤ˻ѤΰҡפˤȤäƤ
//             static ΰѹ. cache ΥҥåΨ򤢤뤿ᡥ
//             ޤС2,3  ®ˤʤ. ʤ
//             äȤ̤ä
//
//  1997/04/26 intelligent Gaussian elimination ʬ
//              matrix 礭򤵤˰̤Ǥ褦
//              5% 餤갵̤뤬ޤ̤ϤʤȤȤ
//
//  1997/07/19
//              Ʊ ǡ  ޡ ̤äե˽񤭹Ǥ
//              θ
//
//  1997/08/06
//              ƱǡޡݡǤ˥ӥ͡ե
//              񤫤Ƥǡ褦ȤƤ
//
//  1997/08/30
//             (p) (pp) (f) ݡޤʣȹ礻
//             ʤ褦ˤ
//
//  1997/09/18 (p) (pp) (f)ݡɬפʸĿ (f)
//             ʤˤϡη̤˴Ƥ˽ʬǡ
//             򽸤褦ѹϡλ줿ַ̤
//             ե˽񤭽ФƤ
//
//  1998/10/19 reduced matrix ʬʬ䤷
//             reduced matrix ˤ Gaussian elimination ǥ
//              Ȥ᤮ƤΤ
//
//  1999/11/23 Large Prime Variation ǡȤ߹碌礭ȥ礭
//             ۤƥ˥Զ
//
//  2000/08/03   factor base θĿ¤ʤ
//
//  2000/08/03   replace intelligent Gauss elimination by block Lanczos method
//

#ifdef __BORLANDC__
#define WIN32
#endif

#ifdef __BORLANDC__
#include <algorith>
#else
#include <algorithm>
#endif

#include <vector>

#ifdef  WIN32
#include <io.h>
#endif

#include <fstream.h>
#include <stdlib.h>

#define _TEST
#include "Mint.hpp"        // multiple precision integer
#include "Fint.hpp"        // real number
#include "LBit.hpp"
#include <math.h>
#include "Lanczos.hpp"      // block Lanczos method
#include "SMatrix.hpp"      // sparse matrix
#include "PPMPQS.hpp"


#ifdef WIN32
#  ifdef __BORLANDC__
#    include <constrea.h>
#    include <conio.h>
#    include <classlib\StdTempl.h>
#    pragma option -Jg
#  else
     int wherex(void) ;
     int wherey(void) ;
     void clrscr(void) ;
     void gotoxy(int x,int y) ;
#    include <conio.h>
#    include <memory.h>
#  endif
#endif

#include <signal.h>
int  bhit=0 ;
inline int isBreak(void)
{
  return bhit ;
}

//  receive signal
void signal_kbhit(int dmy)      //signal handler when Ctrl+Pause ( Ctrl+Z ) is pressed.
{
  bhit=1 ;
  cout << endl << "wait a few minutes" << flush << endl ;
}

#include "Error.hpp"

#ifdef _MEMMAN
#include "MemMan.hpp"
MemMan Mem ;
#endif

#include "CPUTime.hpp"

using namespace std ;

typedef long LONG2[2] ;
typedef long LONG3[3] ;
typedef unsigned short USHORT4[4] ;
typedef unsigned long ULONG2[2] ;


#define MaxPrimes (1000000)
extern const unsigned long MaxSieveResult=512;
#define MaxSearchLoop        256 // 64
#define MaxCycle             1024 // 512

#define MaxSieveUnit (0x00060000UL)

#define SUCCESS_ACCESS 0
#define ACCESS_NONE 0
unsigned long SieveUnit=MaxSieveUnit ; // to be multiple of 4
void Sieve(PPrimes* Primes,unsigned long nPrimes,unsigned long nPrimes2,
           unsigned long nPowers);
void DivideByBase(unsigned long nPrimes,PPrimes* Primes);
bool BitDecomp(long nPrimes,unsigned long* Primes,Mint& q,unsigned long* v2);
unsigned long KS(const Mint& N,const unsigned long nPrimes) ;  // Knuth Sch\"oppel function


Mint _fx ;
Mint _a ;
Mint _b ;
long _DivResult ;
long _nResult ;
char SieveArray[MaxSieveUnit] ;   // work array for sieving
unsigned long _A[20] ;
unsigned long _B[20] ;
unsigned long _OFF ;
unsigned long nSieveResult=MaxSieveResult ;
unsigned long SieveResult[MaxSieveResult+1] ;
unsigned long Lmax ;
unsigned long Pmax ;
unsigned long Pmax2[2] ;
unsigned long Lmax2[2] ;
unsigned long Pmax4 ;

char Cutoff ;
unsigned long SieveWidth ;
unsigned long SieveWidth2 ;

unsigned long lbyte ;

unsigned long nPrimes ;
unsigned long nPrimes2 ;
unsigned long nPowers ;
unsigned long cache_size=128 ;
long _Kr_tab3[]={0,-1,1} ;              //  -(n/3)   Jacobi symbol

static Mint y ;
Mint& pp=y ;
#ifdef __BORLANDC__
constream con ;
#endif


static long MergeDuplicatedData(long nF) ;
bool BitDecomp(long nPrimes,Mint& T) ;


int PrimeDecomp(long nPrimes,unsigned long* Primes,
                                unsigned long* index,Mint& q) ;
unsigned long Gauss(unsigned long nBit1,unsigned long nBit2,
                    unsigned long nRow,unsigned long** Z1,unsigned long** Z2) ;
void Gauss(::iostream& mpqsmat,unsigned long** M,long mRow,
                     long nRow,long nCol,long size) ;
void Gauss(unsigned long** M,long nRow,long nCol,long size,unsigned long* ipiv) ;
void SieveInit(unsigned long nPrimes,unsigned long nPrimes2) ;
void SetMatrix(::fstream& mpqsfx,unsigned long* ifx,
               unsigned long nX,unsigned long nF,
               unsigned long nPrimes,
               unsigned long* prime);
char* parse_in(istream& is) ;
Mint parse(const char* exp) ;


//
// work files
//
static char primefile[]="./MPQPRM.MPQ" ;
static ::fstream mpqsprime ;

static char rootfile[]="./MPQROOT.MPQ" ;
static ::fstream mpqsroot ;

static char fxfile[]="./MPQSFX.MPQ" ;
static ::fstream mpqsfx ;

static char ixfile[]="./MPQSFXIX.MPQ" ;
static ::fstream mpqsfxix ;

static char ppfxfile[]="./MPQSPPFX.MPQ" ;
static ::fstream mpqsppfx ;

static char pairfile[]="./MPQSPAIR.MPQ" ;
static ::fstream mpqspair ;

static char pppfile[]="./MPQSPPP.MPQ" ;
static ::fstream mpqsppp ;

static char errfile[]="./MPQS.LOG" ;
static ::fstream mpqslog ;

static char mpqsred[]="./MPQS-RED.MPQ" ;
static ::fstream mpqs_red ;

static char mpqscol[]="./MPQS-COL.MPQ" ;
static ::fstream mpqs_col ;

static char mpqsrow[]="./MPQS-ROW.MPQ" ;
static ::fstream mpqs_row ;

static char elemfile[]="./MPQSELEM.MPQ" ;
static ::fstream elem ;

static char parmfile[]="./MPQSPARM.IN" ;
static ::fstream parms ;

static char ansfile[]="./MPQSANS.MPQ" ;
static ::fstream mpqs_ans ;

static char ansfile2[]="./MPQSANS2.MPQ" ;
static ::fstream mpqs_ans2 ;
//
// file structure
//
//  mpqsfx   : fully factored (smooth) data
//             1. value of quadratic polynomial
//             2. sqaure part (a*x+b)^2-N mod N
//
//  mpqsfxix : index ( file pointer ) to mpqsfx
//
//  mpqsppfx : partially factored data
//             1 large primes p1,p2
//             2. value of quadratic polynomial
//             3. square part
//
//  mpqsppp  : index ( file pointer ) to mpqsppfx
//             1. large primes p1,p2             (2 * 4 bytes)
//             2. file pointer fo mpqsppfx       (4 bytes)
//
//             ʬ򤵤줺˻Ĥäʬ p1,p2    ( 4 Х)
//             mpqsppfx ؤΥեݥ ( 4 Х)
//             (p1,p2)򤤤 mpqsppfx ɤߤ˹ԤΤ
//             ݤʤΤǥ֤äƻäƤ.
//
//  mpqspair : smooth relations generated from relations with large primes
//             1. file pointers to mpqsppp
//                file pointer < 0 : index to mpqsppfx
//                file pointer > 0 : index to mpqspair
//
//              (p)(pp) 줿ط
//              ˤʤäĤΥǡؤΥեݥ (4Х*2)
//               󼡴ؿ͡ʿʬϽ񤫤Ƥʤ
//              ΤȤ mpqsppfx ؤΥݥ
//              ΤȤ mpqspair ؤΥݥ
//
//  mpqssqrt : roots of x^2=p mod N ( not used now)
//



Mint _t ;
static Mint pN ;
static Mint sq ;
static TBit U,V,W ;
int nLoop_rho ;
unsigned long _p1,_p2 ;
static long LargePrime(long needF,long& nLPP) ;

long npp[10] ;
int  posy ;
Mint const2(2) ;
char _Copyright[]="PPMPQS Ver 2.8 by S.Tomabechi 1995-2001" ;

///////////////////////////////////////////
//
//  compute Knuth-Schr\"oppel function
//
unsigned long KS(const Mint& N,const unsigned long nPrimes)
{
    unsigned long p,q ;
    unsigned long pX ;
    double fMax ;
    unsigned long pMax ;
    Mint pN ;
    unsigned long j ;
    double f ;

    p=1 ;
    pX=p ;
    fMax=0.0 ;
    pMax=(ilog2(N)+1)/3 ;
    if( pMax>40 ) pMax=40 ;  // up to 40

    for(;;) {

        for(;;) {
            pN=p*N ;
            if( (long(pN)&7)==1 ) break ;            // =1 mod 8 ?
            else p=nextprm(p) ;
        }

        f=2*log(2.0)-log(double(p))/2 ;
        q=2 ;
        for( j=2 ; j<nPrimes ; j++ ) {
            do {                           // search prime q s.t. N is a quadratic residue mod q
                q=nextprm(q)    ;
            } while( Kronecker(pN,q)<0 ) ;
            f+=(Kronecker(pN,q)+1)*log(double(q))/q ;
        }
        if( f>fMax ) { fMax=f ; pX=p ; }
        if( p<pMax ) p=nextprm(p) ;
        else break ;
    }
    return pX ;

}

////////////////////////////////////////////
//
//  define factor base
//
//  nPrimes (I) number of primes
//  N       (I) composite number to be factored
//  Primes  (I) array of primes. to be allocated
//  maxPrimes (O) max prime of factor base
void SetPrimes(const unsigned long nPrimes,const Mint& N,
               PPrimes* Primes,unsigned long& maxPrime)
{
    // decide factor base
    // select primes q s.t. pN is a quadratic residue mod q
    unsigned long p,q,wp ;
    unsigned long j ;
    wp=p=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
        for(;;) {
            q=nextprm(wp) ;
            if( Kronecker(N,q)==-1 ){           // qaudratic residue ?
                wp=q ;                          // not so
                continue ;
            }
            else break ;                        // qaudratic residue
        }
        Primes[j].diff=(unsigned short)(q-p) ;  // set difference of prime and previous prime
        wp=p=q ;
#ifdef MPQS_DEBUG
        if( prmdiv(q)!=1 ) {
            cout << endl << "not prime " << q ;
            Error(not prime) ;
        }
#endif
    }
    maxPrime=p ;

}

////////////////////////////////////////////////////
//
//  set square roots  x^2=N mod p
//
//  nPrimes (I) number of primes
//  N       (I) composite number to be factored
//  pX      (I) multiplier ( Knuth-Schr\"oppel function )
//  Primes  (I/O) array of primes. to be allocated
//
void SetSquareRoot(const unsigned long nPrimes,const Mint& N,const unsigned long pX,
                   PPrimes* Primes)
{
    unsigned long q ;
    unsigned long j ;
    q=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
        q+=(unsigned long)(Primes[j].diff) ;

#ifdef MPQS_DEBUG
        if( Kronecker(N,q)==-1 ) {
            cout << endl << q << flush ;
            Error(not square) ;
        }
#endif
        if( q==pX ) Primes[j].Root=0 ;    // q | N  i.e. (N/q)=0
        else Primes[j].Root=sqrt(N,q) ;   // square root x such that x^2=N mod q

#ifdef MPQS_DEBUG
        if( q!=pX ) {
            if( sqrt(N,q)!=sqrt(N,long(q)) ) {
                sqrt(N,long(q)) ;
                sqrt(N,q) ;
                cout << endl << j << q ;
                Error( long sqrt error) ;
            }
            if( (N-Mint(Primes[j].Root)*Primes[j].Root)%q!=0 ) {
                cout << endl << "sqrt Mint " << sqrt(N,Mint(q)) ;
                cout << endl << "sqrt long " << sqrt(N,long(q)) ;
                cout << endl << "sqrt ulong " << sqrt(N%q,q) ;
                cout << endl << "N mod q " << (N%q) ;

                cout << endl << j << " " << q ;
                cout << endl << (N-Mint(Primes[j].Root)*Primes[j].Root)%q ;
                Error( sqrt error) ;
            }
        }
#endif
    }
}

////////////////////////////////////////////////////
//
//  set logarithm
//
//  nPrimes (I) number of primes
//  pX      (I) multiplier ( Knuth-Schr\"oppel function )
//  Scale   (I) scale factor
//  Primes  (I/O) array of primes. to be allocated
//
void Setlog(const unsigned long nPrimes,const unsigned long pX,const double Scale,
                   PPrimes* Primes)
{

    unsigned long q ;
    unsigned long j ;
    q=0 ;
    for( j=1 ; j<nPrimes ; j++ ) {
        q+=(unsigned long)(Primes[j].diff) ;
        if( q==pX ) Primes[j].logP=char(Scale*log(double(q))/2) ;  // q is multiplier
        else Primes[j].logP=char(Scale*log(double(q))) ;
    }
}


void SetPrimesPowers(const unsigned long nPrimes,
                     const Mint& pN,
                     const unsigned long pX,         // multiplier
                     const double Scale,
                     const unsigned long PPmax,
                     const unsigned long PPPmax,
                     const unsigned long offset,
                     PPrimes* Primes,
                     unsigned long& nPowers)
{
    unsigned long p,q ;
    unsigned long j ;
    unsigned long p1,p2 ;
    double f ;
    Mint x ;
    q=2 ;
    nPowers=0 ;
    for( j=2 ; j<nPrimes ; j++ ) {
        q+=(unsigned long)(Primes[j].diff) ;
        if( q>PPPmax ) break ;
        if( q==pX ) continue ; // pX is the multiplier

        p=q ;
        p1=p ;
        p*=q ;
        f=Scale*log(double(p)) ;
        p2=sqrt(pN,q) ;
        while( p<PPmax ) {
            x=(pN-p2*Mint(p2))/p1 ;
            p2=mod(p2+x*invmodp(2*p2,q)*_p1,p) ;
            f=Scale*log(double(p))-f ;
            if( f<1.0 ) continue    ;
            Primes[offset+nPowers].logP=(char)f ;
            Primes[offset+nPowers].Root=p2 ;
            Primes[offset+nPowers].diff=(unsigned short)p ;
            nPowers++ ;
            p1=p ; p*=q ;
        }

    }

}



void Set2Powers(const unsigned long nPrimes,
                     const Mint& pN,
                     const double Scale,
                     const unsigned long PPmax,
                     PPrimes* Primes,
                     unsigned long& nPrimes2)
{
    unsigned long q ;
    unsigned long j ;
    unsigned long p,p1,p2 ;
    double f ;

    q=2 ;
    p=q ;
    p1=p ;
    p*=q ;
    f=Scale*log(double(p)) ;
    p2=1 ;
    nPrimes2=0 ;
    for(;;) {
        for( j=0 ; j<2 ; j++ ) {
            p1=p ;
            p*=q ;
            p2=mod((pN-p2*Mint(p2))/p1,q) ;
            p2+=p2*p1 ;
        }
        if( p>=PPmax ) break ;
        f=Scale*log(double(p))-f ;
        if( f<1.0 ) continue    ;
        Primes[nPrimes2].logP=(char)f ;
        Primes[nPrimes2].Root=p2 ;
        Primes[nPrimes2].diff=(unsigned short)p ;
        nPrimes2++ ;
    }

}


////////////////////////////////
//
// write factor base to file
//
void WritePrimes(const unsigned long nPrimes,const unsigned long nPowers,
                 const unsigned long nPrimes2,
                 PPrimes* Primes)
{
    unsigned long j ;
    unsigned long p ;
    mpqsprime.open(primefile,ios::out|ios::binary) ;
    mpqsprime.write((char*)&nPrimes,sizeof(unsigned long)) ;
    mpqsprime.write((char*)&nPowers,sizeof(unsigned long)) ;
    mpqsprime.write((char*)&nPrimes2,sizeof(unsigned long)) ;
    p=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
      p+=(unsigned long)(Primes[j].diff) ;
      mpqsprime.write((char*)&p,sizeof(p)) ;
    }
    Primes+=nPrimes ;
    for( j=0 ; j<nPowers ; j++ ) {
      p=(unsigned long)(Primes[j].diff) ;
      mpqsprime.write((char*)&p,sizeof(p)) ;
    }
    Primes+=nPowers ;
    for( j=0 ; j<nPrimes2 ; j++ ) {
      p=(unsigned long)(Primes[j].diff) ;
      mpqsprime.write((char*)&p,sizeof(p)) ;
    }

    mpqsprime.close() ;
}

void WriteSquareRoots(const unsigned long nPrimes,const unsigned long nPowers,
                 const unsigned long nPrimes2,
                 PPrimes* Primes)
{
    unsigned long j ;
    unsigned long p ;
    mpqsroot.open(rootfile,ios::out|ios::binary) ;
    mpqsroot.write((char*)&nPrimes,sizeof(unsigned long)) ;
    mpqsroot.write((char*)&nPowers,sizeof(unsigned long)) ;
    mpqsroot.write((char*)&nPrimes2,sizeof(unsigned long)) ;
    p=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
      p=Primes[j].Root ;
      mpqsroot.write((char*)&p,sizeof(p)) ;
    }
    Primes+=nPrimes ;
    for( j=0 ; j<nPowers ; j++ ) {
      p=Primes[j].Root ;
      mpqsroot.write((char*)&p,sizeof(p)) ;
    }
    Primes+=nPowers ;
    for( j=0 ; j<nPrimes2 ; j++ ) {
      p=Primes[j].Root ;
      mpqsroot.write((char*)&p,sizeof(p)) ;
    }

    mpqsroot.close() ;
}

void ReadSquareRoots(const unsigned long nPrimes,const unsigned long nPowers,
                 const unsigned long nPrimes2,
                 PPrimes* Primes)
{
    unsigned long j ;
    unsigned long p ;
    mpqsroot.open(rootfile,ios::in|ios::binary) ;
    mpqsroot.read((char*)&p,sizeof(unsigned long)) ;
    mpqsroot.read((char*)&p,sizeof(unsigned long)) ;
    mpqsroot.read((char*)&p,sizeof(unsigned long)) ;
    p=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
      mpqsroot.read((char*)&p,sizeof(p)) ;
      Primes[j].Root=p ;
    }
    Primes+=nPrimes ;
    for( j=0 ; j<nPowers ; j++ ) {
      mpqsroot.read((char*)&p,sizeof(p)) ;
      Primes[j].Root=p ;
    }
    Primes+=nPowers ;
    for( j=0 ; j<nPrimes2 ; j++ ) {
      mpqsroot.read((char*)&p,sizeof(p)) ;
      Primes[j].Root=p ;
    }

    mpqsroot.close() ;
}



void ReadPrimes(unsigned long& nPrimes,unsigned long& nPowers,
                 unsigned long& nPrimes2,
                 PPrimes*& Primes,
                 unsigned long& maxPrime)
{
    unsigned long j ;
    unsigned long p ;
    unsigned long q ;
    PPrimes* Ptemp ;
    mpqsprime.open(primefile,ios::in|ios::binary) ;
    mpqsprime.read((char*)&nPrimes,sizeof(unsigned long)) ;
    mpqsprime.read((char*)&nPowers,sizeof(unsigned long)) ;
    mpqsprime.read((char*)&nPrimes2,sizeof(unsigned long)) ;
    Ptemp=Primes=new PPrimes[nPrimes+nPowers+nPrimes2] ;

    Ptemp[0].diff=-1 ;
    Ptemp[1].diff=2 ;
    p=2 ;
    for( j=2 ; j<nPrimes ; j++ ) {
        mpqsprime.read((char*)&q,sizeof(unsigned long)) ;
        Ptemp[j].diff=(unsigned short)(q-p) ;
        p=q ;
    }
    Ptemp+=nPrimes ;
    maxPrime=p ;
    for( j=0 ; j<nPowers ; j++ ) {
        mpqsprime.read((char*)&q,sizeof(unsigned long)) ;
        Ptemp[j].diff=(unsigned short)q ;
    }
    Ptemp+=nPowers ;
    for( j=0 ; j<nPrimes2 ; j++ ) {
        mpqsprime.read((char*)&q,sizeof(unsigned long)) ;
        Ptemp[j].diff=(unsigned short)q ;
    }

    mpqsprime.close() ;
}

void ReadPrimes(unsigned long& nPrimes,
               unsigned long*& primes)
{
    unsigned long p ;
    mpqsprime.open(primefile,ios::in|ios::binary) ;
    mpqsprime.read((char*)&nPrimes,sizeof(unsigned long)) ;
    mpqsprime.read((char*)&p,sizeof(unsigned long)) ;
    mpqsprime.read((char*)&p,sizeof(unsigned long)) ;
    primes=new unsigned long[nPrimes] ;

    primes[0]=(unsigned long)(-1) ;
    primes[1]=2 ;
    mpqsprime.read((char*)(primes+2),(nPrimes-2)*sizeof(unsigned long)) ;
    mpqsprime.close() ;
}


PPrimes* Primes ;

int main(int argc,char* argv[])
{

    unsigned long* prime ;
    vector<unsigned long*> sol ;
    long kLPP(0),lLPP(0),mLPP(0) ;
    Mint N ;
    Mint d ;
    Mint NN ;
    unsigned long* index ;
    long nF(0) ;
    bool bsame ;
    char* input_str ;
Input_Number:
    bsame=false ;
    nF=0 ;
    kLPP=0 ;
    lLPP=0 ;
    mLPP=0 ;

    if( argc>1 ) {                      // cache size is specified
        unsigned long cache ;
        cache=atoi(argv[1]) ;
        if( cache==0 ) ;
        else {
            if( cache>6*64 ) cache_size=3*cache_size ;   // greather than prepared array
            else cache_size=cache ;
        }
    }
    SieveUnit=cache_size*1024 ;
//    if( parms.bad() ) {
    if( access(parmfile,ACCESS_NONE)!=SUCCESS_ACCESS ) {  // parameter file does not exist
        NN=0 ;
        d=0 ;
    }
    else {
        // get previous parameters
        parms.open(parmfile,ios::in) ;
        parms >> NN ;
        parms >> d ;
    }

    cout << endl << "Input number ( input 0 to exit )" << endl ;
//  cin >> SieveArray ;
    input_str=parse_in(cin) ;
    N=parse(input_str) ;
    if( N==0 ) return 0 ;                // exit


    if( N<0 ) ~N ;
    if( Even(N) ) Error(input number is even) ;


    // primality test
    if( Rabin(N) ) {                                         // test primality
        cout << endl << N << " is probably prime " ;
        if( APRCL(N) ) {
            mpqslog.open(errfile,ios::out|ios::app) ;
            mpqslog << endl << " ========================= " ;
            mpqslog << endl << N << " is prime " ;
            mpqslog.close() ;
            goto Input_Number ;
        }
    }

    CPUTime cputime ;


long nLPP[3]={0,0,0} ;
memset(npp,'\0',10*sizeof(long)) ;

    if( NN!=0 && N==NN ) {      // check whether input number is as same as previous one.
        bsame=true ;            // yes  same number
        parms >> nF ;           // restart siveing
        parms >> kLPP ;
        parms >> lLPP ;
        lLPP=0 ;
        parms >> mLPP ;
        mLPP=0 ;
        parms >> nLPP[0] ;
        nF=nLPP[0] ;
        parms >> nLPP[1] ;
        parms >> nLPP[2] ;
        cputime.Reset() ;
        cputime.Start() ;
        parms >> cputime ;      // add duration time until now
        parms >> cache_size ;
        SieveUnit=cache_size*1024 ;
    }
    else {
        cputime.Start() ;
        if( NN!=0 ) parms.close() ;
	unsigned long e ;
        if( powerdetect(N,d,e) ) {
            if( d!=1 ) {
                cout << endl << "   is power of prime " << d ;
                mpqslog.open(errfile,ios::out|ios::app) ;
                mpqslog << endl << " ========================= by SIQS " ;

                char PC[2] ;
                mpqslog <<  endl << input_str <<" = " ;
//              if( d>N ) Swap(d,N) ;                            // sort
                // primality test
                PC[0]='P' ; mpqslog << "P" ;
                mpqslog << len10(d) << "^" << e  ;
#if 0
            if( APRCL(N) ) { PC[1]='P' ; mpqslog << "P" ; }
            else           { PC[1]='C' ; mpqslog << "C" ; }
            mpqslog << len10(N)  ;
#endif

                mpqslog << endl << PC[0] << len10(d) <<" = " << d ;
//          mpqslog << endl << PC[1] << len10(N) <<" = " << N ;

                mpqslog << endl << "cputime " << cputime ;
                mpqslog.close() ;
//              unlink(parmfile) ;
                delete [] input_str ;
                goto Input_Number ;
            }
        }
        d=SquareTest(N) ;
        if( d>0 ) {
            cout << endl << " is square of " << d ;
            mpqslog.open(errfile,ios::out|ios::app) ;
            mpqslog << endl << " ========================= by SIQS " ;

            char PC[2] ;
            mpqslog <<  endl << input_str <<" = " ;
//          if( d>N ) Swap(d,N) ;                            // sort
            // primality test
            if( APRCL(d) ) { PC[0]='P' ; mpqslog << "P" ; }
            else         { PC[0]='C' ; mpqslog << "C" ; }
            mpqslog << len10(d) << "^2 " ;
#if 0
            if( APRCL(N) ) { PC[1]='P' ; mpqslog << "P" ; }
            else           { PC[1]='C' ; mpqslog << "C" ; }
            mpqslog << len10(N)  ;
#endif

            mpqslog << endl << PC[0] << len10(d) <<" = " << d ;
//          mpqslog << endl << PC[1] << len10(N) <<" = " << N ;

            mpqslog << endl << "cputime " << cputime ;
            mpqslog.close() ;
            delete [] input_str ;
//          unlink(parmfile) ;
            goto Input_Number ;
        }
        d=0 ;                   // start from the beggining
    }

    long j,k ;
    long l ;
    bool found ;
    found=false ;
    unsigned long p,q,pX ;
#ifdef  WIN32
#  ifdef __BORLANDC__
    con.window(1,1,80,45) ;
    con.clrscr() ;
#  else
    clrscr() ;
#  endif
#else
    cout << "\e[1H\e[2J" ;  // clear screen  (for Linux)

#endif

    // decide size of factor base
#ifdef __BORLANDC__
    long nX ;
     nX=min(MaxPrimes,
            int(    1.78
                *expl(
                     sqrtl(
                         logl((long double)N)
                             *logl(logl((long double)N))
                         /12
                        )
                 )
             ) /32*32
         ) ;

#else
    long nX ;
    nX=int( 1.78
                 *exp(
                         sqrt(
                                log((double)N)
                                *log(log((double)N))
                                /12
                            )
                    )
            )/32*32;
    if( nX>MaxPrimes ) nX=MaxPrimes ;
#endif



    nX=nX<56 ? 56:nX ;

    nPrimes=nX-32 ;

    cout << _Copyright ;
    cout << endl << "C" << len10(N) << "=" << N ;

    Primes=new PPrimes[nPrimes+100] ;

    Primes[0].diff=(unsigned short)(-1) ;   // -1 and 2 must be contained in factor base
    Primes[0].logP=0 ;
    Primes[1].diff=2 ;


    // decide a multiplier
    // ǿ¿ʤ褦 N->pN multiplier 
    pX=KS(N,nPrimes) ;

    pN=pX*N ;                    // replace N by N * multiplier

    // decide factor base
    // select primes q s.t. pN is a quadratic residue mod q
    unsigned long maxPrime ;
    if( !bsame ) {
        SetPrimes(nPrimes,pN,Primes,maxPrime) ;
    }
    else {
        delete [] Primes ;
        ReadPrimes(nPrimes,nPowers,nPrimes2,Primes,maxPrime) ;
    }
    Pmax=maxPrime ;

    // decide parameters
#ifdef __BORLANDC__
    long double lnpN=logl((long double)pN) ;
    SieveWidth=(unsigned long)(40000*expl(0.66*logl(lnpN))) ;
#else
    double lnpN=log((double)pN) ;
    SieveWidth=(unsigned long)(40000*exp(0.66*log(lnpN))) ;
#endif
    if( ilog2(N)<224 ) k=4 ;
    else k=2 ;
    SieveWidth=2*(SieveWidth/SieveUnit/2+k-1)*SieveUnit/k ;

    SieveWidth2=SieveWidth<<1 ;

    double MeanValue=log(double(SieveWidth)*SieveWidth*((long double)pN)/2) ;
    double Scale=2.0*255/MeanValue ;

    long PPmax=isqrt(Pmax) ;
    if( PPmax<65536 ) PPmax=65536 ;
    long PPPmax=isqrt(PPmax) ;
    if( PPPmax<100 ) PPPmax=100 ;

    double Success ;
    double RealSuccess ;
    if( len(pN)<266 ) Success=0.15000-0.00019*len(pN) ;
    else Success=(0.11450-0.000143*len(pN))/1.05 ;


    Mint x,w ;

    x=Mint(Pmax)*Pmax ;
    Pmax2[0]=(unsigned long)x ;
    Pmax2[1]=(unsigned long)x(-1) ;

    // these magic numbers are owed to Prof. Kida
#ifdef __BORLANDC__
    Cutoff=char((0.7075+0.0108*logl((long double)N))*log(Pmax)*Scale) ;
#else
    Cutoff=char((0.7075+0.0108*log((long double)N))*log(Pmax)*Scale) ;
#endif
    Lmax=(unsigned long)(Pmax*(1.28+0.40*lnpN)) ;
    if( Lmax<Pmax ) Lmax=Pmax ;
#ifdef __BORLANDC__
    w=Int(exp(1.78*Fint(logl(Lmax)))) ;
#else
    w=Int(exp(1.78*Fint(log(Lmax)))) ;
#endif
    Lmax2[0]=(unsigned long)w ;
    Lmax2[1]=(unsigned long)w(-1) ;
    nLoop_rho=isqrt(Lmax)/100 ;    // max loop number of rho-method

    cout << endl << "#FactorBase " << nPrimes ;
    cout << " max FactorBase " << Pmax ;
    cout << endl << "max LargePrime " << Lmax ;
    cout << " Upper Bound " << w ;
    cout << endl << "SieveWidth " << SieveWidth  ;
    cout << " SieveUnit " << SieveUnit ;
    cout << " CutOff " << (int)Cutoff ;
    cout << " multiplier " << pX ;

    // calculate square roots of the equation X*X=pN mod q
    if( !bsame ){
        SetSquareRoot(nPrimes,pN,pX,Primes) ;
    }
    else {
        ReadSquareRoots(nPrimes,nPowers,nPrimes2,Primes) ;
    }

    // set Scale*log_2(P)
    Setlog(nPrimes,pX,Scale,Primes) ;

    if( ilog2(pN)>256 && !bsame ) {
      SetPrimesPowers(nPrimes,
                     pN,
                     pX,         // multiplier
                     Scale,
                     PPmax,
                     PPPmax,
                     nPrimes,
                     Primes,
                     nPowers) ;

      Set2Powers(nPrimes,
                     pN,
                     Scale,
                     PPmax,
                     Primes+nPrimes+nPowers,
                     nPrimes2) ;

    }


    if( !bsame ) {
        WritePrimes(nPrimes,nPowers,nPrimes2,Primes) ;
        WriteSquareRoots(nPrimes,nPowers,nPrimes2,Primes) ;
    }


    // open work files
    mpqsfx.open(fxfile,ios::binary|ios::out|ios::in) ;
    if( mpqsfx.bad() ) {
        cerr << endl << "Error : could not open file " << fxfile ;
        Error(file not found) ;
    }

    mpqsfxix.open(ixfile,ios::binary|ios::in|ios::out) ;
    if( mpqsfxix.bad() ) {
        cerr << endl << "Error : could not open file " << ixfile ;
        Error(file not found) ;
    }

    mpqsppfx.open(ppfxfile,ios::binary|ios::in|ios::out) ;
    if( mpqsppfx.bad() ) {
        cerr << endl << "Error : could not open file " << ppfxfile ;
        Error(file not found) ;
    }

    mpqspair.open(pairfile,ios::binary|ios::in|ios::out) ;
    if( mpqspair.bad() ) {
        cerr << endl << "Error : could not open file " << pairfile ;
        Error(file not found) ;
    }
    mpqspair.seekp(8) ;

    mpqsppp.open(pppfile,ios::binary|ios::in|ios::out) ;
    if( mpqsppp.bad() ) {
        cerr << endl << "Error : could not open file " << pppfile ;
        Error(file not found) ;
    }

    if( d==0 ) {
        d=Int(sqrt(sqrt(Fint(pN))/SieveWidth*738/1000)) ;
        if( d<(unsigned long)Pmax+3 ) d=(unsigned long)Pmax+3 ;
        d=(d/4)*4+3 ;                    // d=3 mod 4

        mpqsppfx.write((char*)&_p1,sizeof(_p1)) ; // write dummy data
        mpqsppfx.write((char*)&_p2,sizeof(_p2)) ; // write dummy data
        write(mpqsppfx,_fx) ;
        write(mpqsppfx,sq) ;
    }
    else {
        parms >> Success ;

        parms >> l ;
        for( k=0,j=0 ; j<nF ; j++ ) {
            l=mpqsfx.tellp() ;
            read(mpqsfx,_fx) ;
            read(mpqsfx,sq) ;
#ifdef MPQS_DEBUG
if( !BitDecomp(_nPrimes,_fx) ) {
    PrimeDecomp(_nPrimes-1,Primes+1,(unsigned long*)SievePos,_fx) ;
    cout << endl << "j " << j ;
    cout << endl << "_fx " << _fx ;
    Error(bit decomposition error) ;
}

#endif
            k++ ;
            mpqsfxix.write((char*)&l,sizeof(l)) ;
        }
        nF=k ;

        parms >> l ;

        parms >> l ;
        mpqsppfx.read((char*)&_p1,sizeof(long)) ; // read dummy data
        mpqsppfx.read((char*)&_p2,sizeof(long)) ; // read dummy data
        read(mpqsppfx,_fx) ;
        read(mpqsppfx,sq) ;
        for( j=0 ; j<kLPP ; j++ ) {
            l=mpqsppfx.tellp() ;
            l=-l ;
            mpqsppfx.read((char*)&_p1,sizeof(long)) ;
            mpqsppfx.read((char*)&_p2,sizeof(long)) ;
            mpqsppp.write((char*)&_p1,sizeof(long)) ;
            mpqsppp.write((char*)&_p2,sizeof(long)) ;
            mpqsppp.write((char*)&l,sizeof(l)) ;

            read(mpqsppfx,_fx) ;
            read(mpqsppfx,sq) ;

        }

        parms >> l ;
        parms >> l ;
        parms >> l ;
        if( !parms.eof() ) SieveUnit=l ;
        parms.close() ;
    }

#ifdef  WIN32
    cout << endl << "To stop sieving, press Ctrl + Pause" ;
#else
    cout << endl << "To stop sieving, press Ctrl + Z" ;
    cout << endl ;
#endif

    //SIGBREAK
#ifdef WIN32
//cout << endl << "signal handeler " ;
    signal(SIGBREAK,signal_kbhit) ;
//    signal(SIGBREAK,signal_kbhit) ;
#else
    // set signal handler (Linux version)
    signal(SIGTSTP,signal_kbhit) ;
#endif

    Mint c,a2,d2 ;
    long newFF(0) ;
    Mint const3(3) ;
    cout << endl << endl;

#ifdef WIN32
# ifdef __BORLANDC__
    posy=con.rdbuf()->wherey() ;
# else
    posy=wherey() ;
# endif
#endif



    for(;;) {
                            // make a quadratic polynomial a*x^2+b*x+c
                            // decide d which is probably prime and
                            //   d=3 mod 4 (pN/d)=1
                            //   Euler's criterion is satisfied by d

                            // ¿༰    a*x^2+b*x+c
                            // d=3 mod 4 (pN/d)=1 ǿ餷
                            // d  Euler's criterion Τ
        d += 4UL ;
                            // discard d which has a small factor
#if 0
        if( gcd(d.isRemUL(111546435UL),111546435UL)!=1 ) continue ; // 111546435=3*5*7*11*13*17*19*23

#else
        if( gcd(d,1033715657UL)!=1 ) continue ; // 1033715657=13*23*37*41*43*53
        if( gcd(d,1035563755UL)!=1 ) continue ; // 1035563755=5*17*29*61*67*97
        if( gcd(d,1036405977UL)!=1 ) continue ; // 1036405977=3*19*47*59*79*83
        if( gcd(d,1039058713UL)!=1 ) continue ; // 1039058713=7*11*31*67*73*89

#endif

        if( Kronecker(pN,d)!=1 ) continue ;  // pN is not a quadratic residue mod d
        _fx.isShiftr(w.isSubUL(d,1),1) ;     //  _fx=(d-1)/2
                                        //  2^((d-1)/2) mod d = (2/d) ?
                                        // Euler's criterion w.r.t base=2
        if( long(_a.apower(const2,_fx,d))!=_Kr_tab[((unsigned long)d)&7] )
            continue ;
                                        //  3^((d-1)/2) mod d = (3/d) ?
                                        // Euler's criterion w.r.t base=3
                                        //  (3/d)=-(d/3)=-(d%3/3) because d=3 mod 4
        if( long(_a.apower(const3,_fx,d))!=_Kr_tab3[d%3] ) continue ;

        _a.power(pN,(d-3)>>2,d) ;
        _b.isRem(Mint::_Mtemp.isMul(pN,_a),d) ;
        c.isRem((((pN-_b*_b)/d)*_a*(d+1)/2),d) ;
        _a.isMul(d,d) ;
        a2.isShiftl(_a,1) ;
        d2.isShiftl(d,1) ;
        d2.isInvModp(d2,pN) ;
        _b.isRem((Mint::_Mtemp.isMul(c,d)).isAdd(Mint::_Mtemp,_b),_a) ;
        if( (((unsigned long)_b)&1)==0 )  (_b.isSub(_b,_a)).nDim&=_POSITIVE_ ;
                                                             // b is even here
        w.isShiftl(_a,2) ;                  // w=4*_a
        divide(Mint::_Mtemp.isSub(_b*_b,pN),w,c,y) ;
        if( !y.isZero() ) continue ;

        y=-_b/a2-SieveWidth ;
                                                            // sieve
        Sieve(Primes,nPrimes,nPrimes2,nPowers) ;

                                            // _nResult : number of candidates after sieving
        for( ; _nResult>0 ;  )  {

            _OFF=SieveResult[--_nResult] ;      // offset in seive array
            x=y+_OFF ;

            ((_fx.isMul(_a,x)+=_b)*=x)+=c ;     //  (a*x^2+b*x+c)
            if( _fx.isZero() ) continue ;

#if 0
{
  unsigned long p=2 ;
  for( int j=2 ; j<nPrimes ; j++ ) {
    p+=(unsigned long)(Primes[j].diff) ;
    if( mod(_OFF-Primes[j].SievePos[0],p)==0 ) {
      if( _fx%p!=0 ) {
        cout << endl << _OFF << " " << p ;
        Error(_fx%p!=0) ;
      }
    }
    if( mod(_OFF-Primes[j].SievePos[1],p)==0 ) {
      if( _fx%p!=0 ) {
        cout << endl << _OFF << " " << p  ;
        Error(_fx%p!=0) ;
      }
    }
  }
}
#endif
            DivideByBase(nPrimes,Primes) ;   // divide by factor base

            if( _DivResult<0 ) continue ;  // not smooth

            nLPP[_DivResult]++ ;
            found=true ;
            sq=mod((a2*x+_b)*d2,pN) ;      // sq=(ax+b)^2-N
            if ( _DivResult==0 ) {         // fully factored
                l=mpqsfx.tellp() ;
                mpqsfxix.write((char*)&l,sizeof(l)) ;
                write(mpqsfx,_fx) ;
                write(mpqsfx,sq) ;
                nF++ ;

#ifdef MPQS_DEBUG
{
unsigned long nPrimes ;
unsigned long* primes ;
ReadPrimes(nPrimes,primes) ;
if( !BitDecomp(nPrimes,primes,x=_fx,v2) ) {
    cout << endl << "d " << d  ;
    delete [] primes ;
    Error(bit decomposition error) ;
}
delete [] primes ;
}
#endif

            }
            else {                                   //  smooth but large prime(s)
                l=-mpqsppfx.tellp() ;
                mpqsppp.write((char*)&_p1,sizeof(_p1)) ;
                mpqsppp.write((char*)&_p2,sizeof(_p2)) ;
                mpqsppp.write((char*)&l,sizeof(l));  // file pointer to mpqsppfx

                mpqsppfx.write((char*)&_p1,sizeof(_p1)) ;
                mpqsppfx.write((char*)&_p2,sizeof(_p2)) ;
                write(mpqsppfx,_fx) ;
                write(mpqsppfx,sq) ;
                kLPP++ ;

#ifdef MPQS_DEBUG
{
x=_fx ;
(x/=_p1)/=_p2 ;
unsigned long nPrimes ;
unsigned long* primes ;
ReadPrimes(nPrimes,primes) ;
if( !BitDecomp(nPrimes,primes,x,v2) ) {
    cout << endl << "d " << d  ;
    DivideByBase(nPrimes,Primes) ;
    delete [] primes ;
    Error(bit decomposition error) ;
}
delete [] primes ;
}
#endif
            }
        }

        if( found    /* && kLPP>lLPP */ ) {
            found=false ;
#ifdef  WIN32
#  ifdef __BORLANDC__
        con.rdbuf()->gotoxy(1,posy) ;
        cout << endl    << nF << "("
         << int(nF+newFF+Success*(kLPP-lLPP)) << ")/" << nX << " ";
#  else
        gotoxy(1,posy) ;
        cout << endl    << nF << "("
         << int(nF+newFF+Success*(kLPP-lLPP)) << ")/" << nX << " "  ;

#  endif
#else
        cout << "\e[1A\e[1G" ;          // move cursor up by 1 line (for Linux)
                                        //ԾƬˤäƤ
cout << endl    << nF << "("
         << int(nF+newFF+Success*(kLPP-lLPP)) << ")/" << nX << " "  << flush ;
#endif
                                         // kLPP : # of partially factored data
            if( nF+newFF+Success*(kLPP-lLPP)>nX ) {
cout << endl <<  "search cycles "  << flush   ;
//                lLPP=kLPP-lLPP ;
                // generate smooth data from large primes
                if( nX>nF ) {
                    newFF=LargePrime(nX-nF,kLPP) ;
#if 0
                    Success/=(1.0-0.4) ;         // modify estimation
                    while( Success>=0.2 ) Success*=0.7 ;
                    lLPP=kLPP ;
                    mLPP=kLPP ;
#endif
                    if( newFF>0 ) {
                        nF=MergeDuplicatedData(nF+newFF) ;
                        newFF=0 ;
                    }
                    else {
                        Success/=(1.0+0.05) ;   // modify estimantion
                        newFF=-newFF ;
                    }

                }
                if( nF>nX ) break ;        // found enough data
                else {
cout << endl << (nF+newFF) << " data gained."
         << " try more. " << flush ;
            newFF=0 ;
#ifndef WIN32
        cout << "\e[1A\e[1G" ;          // move cursor up to 1 line  (for Linux)
        cout << "\e[1A\e[1G" ;
#endif
                }

            }
        }
        if( isBreak() ) break ;

    }

    posy++ ;
//con.rdbuf()->gotoxy(1,posy) ;
    cout << endl << "(f)  type " << nLPP[0] << " total " << nF ;
    posy++ ;
    cout << endl << "(p)  type " << nLPP[1]
         << "( p_p " << npp[0] << ", p_p_pp " << npp[1]
         << ", p_p_p_p_pp_pp " << npp[4] <<  ")" ;
    posy++ ;
    cout << endl << "(pp) type " << nLPP[2]
         << "( pp_pp " << npp[2] << ", pp_pp_pp " << npp[3] << ",other "<< npp[9] <<")" << flush;
//cout << endl << "enter LargePrime "<< enterLP ;
q=0 ;
for( j=0 ; j<10 ; j++ ) q += npp[j] ;
//posy++ ;

    if(  kLPP!=0 ) RealSuccess=double(nX-nLPP[0])/kLPP ;

{
    parms.open(parmfile,ios::out) ;

    parms << endl << N ;
    parms << endl << d ;
    parms << endl << nF ;
    parms << endl << (nLPP[1]+nLPP[2]) ;
    parms << endl << lLPP ;
    parms << endl << mLPP ;
    parms << endl << nLPP[0] ;
    parms << endl << nLPP[1] ;
    parms << endl << nLPP[2] ;
    cputime.Stop() ;
    parms << endl << cputime ;
    parms << endl << cache_size ;

    parms << endl << Success ;
    l=mpqsfx.tellp() ;
    parms << endl << l ;

    l=mpqsfxix.tellp() ;
    parms << endl << l ;

    l=mpqsppfx.tellp() ;
    parms << endl << l ;

    l=mpqsppp.tellp() ;
    parms << endl << l ;

    l=mpqspair.tellp() ;
    parms << endl << l ;

    parms << endl << SieveUnit  << endl ;
    parms.close() ;
}

    mpqsppfx.close() ;
    mpqspair.close() ;
    mpqsppp.close() ;
    mpqsfxix.close() ;
    delete [] Primes ;
    if( nF<nX ) goto RET ;


    if( nX>MaxPrimes ) nX=MaxPrimes ;

    unsigned long* ifx ;                // file pointers to all smooth data
    ifx=new unsigned long[nX] ;

    mpqsfxix.open(ixfile,ios::binary|ios::in) ;
    if( mpqsfxix.bad() ) {
        cerr << endl << "Error : could not open file " << ixfile ;
        Error(file not found) ;
    }
    mpqsfxix.read((char*)ifx,nX*sizeof(unsigned long)) ;
    mpqsfxix.close() ;

    ReadPrimes(nPrimes,prime) ;
    SetMatrix(mpqsfx,ifx,nX,nF,nPrimes,prime) ;
    delete [] prime ;
    mpqsfx.close() ;

    long heavy_col ;
posy++ ;
///////////////////////////////////////////////////////////////////
//
// block Lanczos method
//

    cout << endl << endl << "block Lanczos method " << flush ;

RETRY_LANCZOS:

    SparseMatrix* SM ;                         // sparse matrix
    unsigned long* v ;                         // bit vector
    elem.open(elemfile,ios::binary|ios::in) ;
    SM=new SparseMatrix(nX,nPrimes) ;          // make sparse matrix from elements file
    read(elem,*SM,nX,nPrimes) ;
    elem.close() ;

    v=new unsigned long[(nX+31)/32] ;          // allocate bit vector
    memset(v,0x00,(nPrimes+31)/32*4) ;         // set zero as vector
    V.SetBitString(nPrimes,v) ;

    unsigned long* old_row ;
    double Mreduction ;
    unsigned long nSingleton ;

    old_row=new unsigned long[SM->nrow] ;
    for( j=0 ; j<SM->nrow ; j++ ) old_row[j]=j ;
    nSingleton=SM->nrow ;
    SM->DeleteSingleton(old_row) ;             // delete singleton rows
    nSingleton-=SM->nrow ;

#ifdef NFS_DEBUG
    cout << endl << "post deleting singleton " ;
    SM->statics(cout) ;
#endif

    Mreduction=SM->NumberOfNonZeroEntry() ;

    unsigned long* new_col ;
    new_col=new unsigned long[nPrimes] ;

    memset(new_col,0x00,nPrimes*sizeof(unsigned long)) ;

    ULONG2* weight_index ;
    weight_index=new ULONG2[nPrimes] ;
    for( j=0 ; j<nPrimes ; j++ ) {
        weight_index[j][0]=SM->w_col[j] ;       // set column weight
        weight_index[j][1]=j ;                  // original column number
    }

    QSortULD(nPrimes,8,0,(unsigned long*)weight_index) ;
    for( k=0,j=nPrimes-1 ; ; j-- ) {
        if( weight_index[j][0]!=0 ) break ;
        new_col[weight_index[j][1]]=SparseMatrix::isLight ;  // column to be used
        k++ ;
    }
    cout << endl << "#singleton " << nSingleton << " : zero-weight columns " << (k-nSingleton) ;
    for( k=0 ;; j-- ) {
        if( weight_index[j][0]!=1 ) break ;   // 󤫤
        new_col[weight_index[j][1]]=SparseMatrix::isDelete ;    // column to be deleted
        k++ ;
    }

    if( k!=0 ) cout << ": deleted columns " << k ;

    unsigned long denseColumn ;
#if 0
    if( SM->memory()>16*1024 ) denseColumn=20 ;
    else denseColumn=0 ;
#endif
    denseColumn=0 ;

    // delete some columns with heavy weight to save memory
    // but denseColumn=0 now
    for( j=0 ; j<denseColumn ; j++ ) {
        new_col[weight_index[j][1]]=SparseMatrix::isHeavy ; // columns with many non-zero entries
    }
    delete [] weight_index ;

    for( k=0,j=0 ; j<nPrimes ; j++ ) {
        if( new_col[j]<SparseMatrix::isDelete ) {
            new_col[j]=k ;
            k++ ;
        }
    }
    unsigned long delcol ;
    delcol=SM->DeleteColumn(new_col) ;
    Mreduction=Mreduction-delcol ;


    for( k=0,j=0 ; j<nPrimes ; j++ ) {
        if( new_col[j]==SparseMatrix::isHeavy ) {  // column with many non-zero entries
            new_col[j]=k ;
            k++ ;
        }
        else new_col[j]=SparseMatrix::isHeavy ;
    }
    heavy_col=k ;     // number of columns removed from matrix (= denseColumn )

    mpqs_col.open(mpqscol,ios::binary|ios::out) ;
    mpqs_col.write((char*)&nPrimes,sizeof(unsigned long)) ;
    mpqs_col.write((char*)new_col,nPrimes*sizeof(unsigned long)) ;
    mpqs_col.close() ;
    delete [] new_col ;

    SM->DeleteNullRow(old_row) ;       // delete rows without non-zero entry

    mpqs_row.open(mpqsrow,ios::binary|ios::out) ;
    mpqs_row.write((char*)&(SM->nrow),sizeof(unsigned long)) ;
    mpqs_row.write((char*)old_row,(SM->nrow)*sizeof(unsigned long)) ;
    mpqs_row.close() ;

    mpqs_red.open(mpqsred,ios::binary|ios::out) ;
    write(mpqs_red,*SM) ;             // write reduced matrix
    mpqs_red.close() ;
    mpqs_red.open(mpqsred,ios::binary|ios::in) ;

    // reload reduced matrix
#ifdef LANCZOS_TRANSPOSE
    read(mpqs_red,*SM,SM->nrow,SM->ncol) ;
#else
    readTransposed(mpqs_red,*SM,SM->ncol,SM->nrow) ;
#endif

    mpqs_red.close() ;
    unlink(mpqsred) ;
    delete [] SM->w_col ;   // column weight are not needed any more
    SM->w_col=NULL ;

    Lanczos* L ;
    L=new Lanczos(SM) ;     // make Lanczos scheme

    unsigned long Nentry ;
    double entry_per_row;

    unsigned long SMatrixMemory ;

#ifdef LANCZOS_TRANSPOSE
    SMatrixMemory=L->BT->memory() ;          // memeory size of matrix
    Nentry=L->BT->NumberOfNonZeroEntry() ;   // number of non-zero entries
    entry_per_row=L->BT->entry_per_row() ;
    Mreduction=Nentry/Mreduction*100.0 ;
    L->BT->statics(cout) ;
#else
    SMatrixMemory=L->B->memory() ;
    Nentry=L->B->NumberOfNonZeroEntry() ;
    entry_per_row=L->B->entry_per_row() ;
    Mreduction=Nentry/Mreduction*100.0 ;
    (L->B)->statics(cout) ;
#endif
    if( denseColumn!=0 ) {
        cout << " " << Mreduction << "% compressed " ;
    }

    unsigned long LanczosLoop ;
    unsigned long msol ;
    unsigned long reducedFF ;
    unsigned long nrel ;
    // search non-trivial relations
    for( LanczosLoop=1 ;; LanczosLoop++ ) {
        msol=L->Solve(sol) ;            // search solutions s.t. B*X=0  B=A*A^T
        cout << endl ;
        L->statics(cout) ;
        cout <<  " gained " << msol << " solutions " ;
        if( msol>heavy_col) break ;
    }

#ifdef LANCZOS_TRANSPOSE
    reducedFF=L->BT->nrow ;
#else
    reducedFF=L->B->ncol ;
#endif
    delete L ;
    delete SM ;
    nrel=sol.size() ;


    mpqs_ans.open(ansfile,ios::binary|ios::out) ;
    mpqs_ans.write((char*)&reducedFF,sizeof(unsigned long)) ;
    for( j=0 ; j<nrel ; j++ ) {
        mpqs_ans.write((char*)sol[j],((reducedFF+31)/32)*sizeof(unsigned long)) ;
    }
    mpqs_ans.close() ;
    // Ƭ msol Ĥθ ۤΤΤɬפʤ
    while( msol<sol.size() ) {
        delete [] sol.back() ;
        sol.pop_back() ;
    }

    unsigned long** QcMat ;
    QcMat=(unsigned long**)SieveArray ;
    unsigned long** PseudoSol ;
    PseudoSol=QcMat+1024 ;
    for( j=0 ; j<msol ; j++ ) {
        QcMat[j]=new unsigned long[(heavy_col+31)/32] ;
        memset(QcMat[j],0x00,(heavy_col+31)/32*4) ;
    }


    U.SetBitString(heavy_col,v) ;
    mpqs_col.open(mpqscol,ios::binary|ios::in) ;
    mpqs_col.read((char*)&l,sizeof(unsigned long)) ;
    new_col=new unsigned long[nPrimes] ;
    mpqs_col.read((char*)new_col,nPrimes*sizeof(unsigned long)) ;
    mpqs_col.close() ;

    elem.open(elemfile,ios::binary|ios::in) ;

    // reconstruct matrix from all columns
    // 󤫤 column Ȥ
    unsigned long irow ;
    unsigned long n ;
    for( irow=0,j=0 ; j<reducedFF ; j++ ) {
        memset(v,0x00,(heavy_col+31)/32*4) ;
        for(;; irow++ ) {
            elem.read((char*)&n,sizeof(unsigned long)) ;  // # non-zero entries
            if( old_row[j]==irow ) {
                irow++ ;
                break ;
            }
            for( k=0 ; k<n ; k++ )
                elem.read((char*)&l,sizeof(unsigned long)) ;  // nonzero entry number
        }
        for( k=0 ; k<n ; k++ ) {
            elem.read((char*)&l,sizeof(unsigned long)) ;  // nonzero entry number
            if( new_col[l]<SparseMatrix::isDelete ) {
                U.BitOn(new_col[l]) ;
            }
        }
        for( k=0 ; k<msol ; k++ ) {
            if( BitIndex(sol[k],j) ) {
                V.SetBitString(heavy_col,QcMat[k]) ;
                if( heavy_col!=0 ) V^=U ;
            }
        }
    }
    delete [] old_row ;
    elem.close() ;

    nrel=sol.size() ;
    for( j=0 ; j<msol ; j++ ) PseudoSol[j]=sol[j] ;

    unsigned long nsol ;
    // find real dependencies
    // create solutions of A^T*X=0 from solutions of (A*A^T)*X=0
    if( heavy_col!=0 ) nsol=Gauss(heavy_col,reducedFF,msol,QcMat,PseudoSol) ;
    else nsol=0 ;

#if 0
cout << endl << "Rank " << nsol ;
for( j=nsol  ; j<sol.size() ; j++ ) {
  V.SetBitString(heavy_col,QcMat[j]) ;
   cout << endl << V ;
}
#endif

    for( j=0 ; j<sol.size() ; j++ ) delete [] QcMat[j] ;
    // write solutions to file
    mpqs_ans2.open(ansfile2,ios::binary|ios::out) ;
    mpqs_ans2.write((char*)&reducedFF,sizeof(unsigned long)) ;

    for( j=0 ; j<sol.size() ; j++ ) sol[j]=PseudoSol[j] ;
    reverse(sol.begin(),sol.end()) ;
    for( j=0 ; j<nsol ; j++ ) {
        delete [] sol.back() ;
        sol.pop_back() ;
    }

    nsol=sol.size() ;


#ifdef MPQS_CHECK_SOLUTION                 // check solutions
    {
        int j,jj ;
        for( k=0,j=0 ; j<nPrimes ; j++ ) {
            if( new_col[j]!=SparseMatrix::isDelete ) {
                new_col[j]=k ;
                k++ ;
            }
        }
        mpqs_row.open(mpqsrow,ios::binary|ios::in) ;
        mpqs_row.read((char*)&j,sizeof(unsigned long)) ;
        old_row=new unsigned long[reducedFF] ;
        mpqs_row.read((char*)old_row,reducedFF*sizeof(unsigned long)) ;
        mpqs_row.close() ;
        V.SetBitString(nPrimes,(unsigned long*)SieveResult) ;
        U.SetBitString(nPrimes,v) ;
        unsigned long m ;
        for( l=0 ; l<nsol ; l++ ) {
            elem.open(elemfile,ios::binary|ios::in) ;
            memset(SieveResult,0x00,(nPrimes+31)/32*4) ;
            for( irow=0,jj=0 ; jj<reducedFF ; jj++ ) {
                memset(v,0x00,(nPrimes+31)/32*4) ;             // clear U
                for( ;; irow++ ) {
                    elem.read((char*)&n,sizeof(unsigned long)) ;        // # entries
                    if( old_row[jj]==irow ) {
                        irow++ ;
                        break ;
                    }
                    else {
                        for( k=0 ; k<n ; k++ ) {
                            elem.read((char*)&m,sizeof(unsigned long)) ;    // nonzero entry index
                        }
                    }
                }
                for( k=0 ; k<n ; k++ ) {
                    elem.read((char*)&m,sizeof(unsigned long)) ;    // nonzero entry index
                    if( new_col[m]!=SparseMatrix::isDelete ) {
                        U.BitOn(new_col[m]) ;
                    }
                }
                if( BitIndex(sol[l],jj) ) V^=U ;
            }
            elem.close() ;
            for( k=0 ; k<(nPrimes+31)/32 ; k++ ) {
                if( SieveResult[k]!=0 ) {
//                  cout << endl << V ;
                    delete [] sol[l] ;
                    sol[l]=NULL ;
                    Error(why) ;
                    cout << k << "why " ;
                    break ;
                }
            }
        }
        delete [] old_row ;
    }
#endif


    delete [] new_col ;
    j=nsol ;
    unsigned long realsol ;
    for( realsol=0,k=0 ; k<sol.size() ; k++ ) {
        if( sol[k] ) realsol++ ;
    }
    nsol=realsol ;
    mpqs_ans2.write((char*)&nsol,sizeof(unsigned long)) ;
    for( j=0 ; j<sol.size() ; j++ ) {
        if( sol[j] ) mpqs_ans2.write((char*)sol[j],((reducedFF+31)/32)*sizeof(unsigned long)) ;
    }
    mpqs_ans2.close() ;
    for( j=0 ; j<sol.size() ; j++ ) {
        if( sol[j] ) delete [] sol[j] ;
    }

    while( !sol.empty() ) sol.pop_back() ;

    sol.resize(0) ;

    mpqs_row.open(mpqsrow,ios::binary|ios::in) ;
    mpqs_row.read((char*)&j,sizeof(unsigned long)) ;
    old_row=new unsigned long[reducedFF] ;
    mpqs_row.read((char*)old_row,reducedFF*sizeof(unsigned long)) ;
    mpqs_row.close() ;

    mpqs_ans2.open(ansfile2,ios::binary|ios::in) ;
    mpqs_ans2.seekp(0) ;
    mpqs_ans2.read((char*)&reducedFF,sizeof(unsigned long)) ;
    mpqs_ans2.read((char*)&nsol,sizeof(unsigned long)) ;

    lbyte=((reducedFF+31)>>5)<<2 ;

////////////////////////////////////////////////////////

    mpqsfxix.open(ixfile,ios::binary|ios::in) ;
    mpqsfxix.read((char*)ifx,nX*sizeof(long)) ;
    mpqsfxix.close() ;

    mpqsfx.open(fxfile,ios::binary|ios::in) ;

    V.SetBitString(reducedFF,v) ;

#ifdef  WIN32
#ifdef __BORLANDC__
posy=con.rdbuf()->wherey() ;
#else
posy=wherey() ;
#endif
#endif

#ifdef MPQS_DEBUG
//  mpqsfx.open(fxfile,ios::binary|ios::in) ;
    ReadPrimes(nPrimes,prime) ;
    unsigned long* iindex ;
    iindex=new unsigned long[nPrimes] ;
    for( j=0 ; j<reducedFF ; j++ ) {
        mpqsfx.seekp(ifx[old_row[j]]) ;
        read(mpqsfx,_fx) ;
        read(mpqsfx,sq) ;
        if( !PrimeDecomp(nPrimes-1,prime+1,iindex+1,_fx) ){
          Error(PrimeDecomp error) ;
        }
    }
//  mpqsfx.close() ;
    for( j=0 ; j<reducedFF ; j++ ) {
      cout << " " << old_row[j] ;
    }
    delete [] iindex ;
    delete [] prime ;
#endif


//////////////////////////////////////////
//
//  final stage ( find a non-trivial factor )
//

    // set factor base
    ReadPrimes(nPrimes,prime) ;
    index=new unsigned long[nX] ;
    for( l=0 ; l<nsol ; l++ ) {
        mpqs_ans2.read((char*)v,lbyte) ;   // read a solution ( bit vector )
        x=1 ;

        for( k=0 ; k<nPrimes ; k++ ) index[k]=0 ;
        for( k=0 ; k<reducedFF ; k++ ) {
            if( V[k] ) {
                mpqsfx.seekp(ifx[old_row[k]]) ;
                read(mpqsfx,_fx) ;
                read(mpqsfx,sq) ;                    // square part
                (x*=sq)%=N  ;                        // x=mod(x*sq,pN)
                if( _fx<0 ) ~_fx ;
                                        // primary decomposition by factor base
                if( !PrimeDecomp(nPrimes-1,prime+1,index+1,_fx) ){
#if 1
                    break ;
#else
                    Error(Prime decomposition error) ;
#endif
                }
            }
        }
        for( y=1,k=1 ; k<nPrimes ; k++ ) {
            p=prime[k] ;
            if( index[k]!=0 ) {

#ifdef MPQS_DEBUG
if( index[k]&1 ) Error(not square) ;
#endif
                q=index[k]>>1 ;
                y*=power(sq=p,_fx=q,N) ; // multiply by p^q mod N
                y%=N ;
            }
        }

        d=gcd(x-y,N) ;
        if( d!=1 && d!=N && d!=0 ) goto RET2 ; // find a factor
        cout << endl << "failed " ;
        cout << " gcd " << d << " " ;
    }

  delete [] prime ;                       // failed to find a factor
  delete [] index ;
  delete [] old_row ;
  delete [] v ;
  mpqs_ans2.close() ;
  mpqsfx.close() ;
  goto RETRY_LANCZOS ;

RET2:
  delete [] prime ;
  delete [] index ;
  delete [] ifx ;
  delete [] old_row ;
  delete [] v ;

RET: ;
    mpqs_ans2.close() ;
#if 0
    mpqsfx.seekp(0) ;
    for( j=0 ; j<nF ; j++ ) {
        read(mpqsfx,_fx) ;
        read(mpqsfx,_fx) ;
    }
#endif
    mpqsfx.close() ;
    if( nF>nX ) {
        cout << endl << "found  factor " << d ;
        cout << endl << "cputime " << cputime ;

        mpqslog.open(errfile,ios::out|ios::app) ;
        mpqslog << endl << " ========================= " ;
        mpqslog << endl << "#FB " << nX ;
        mpqslog << endl << "(f)  type " << nLPP[0] ;
        mpqslog << endl << "(p)  type " << nLPP[1]
         << "( p_p " << npp[0] << ", p_p_pp " << npp[1]
         << ",p_p_p_p_pp_pp " << npp[4] << ")" ;
        mpqslog << endl << "(pp) type " << nLPP[2]
             << "( pp_pp " << npp[2] << ", pp_pp_pp " << npp[3] << ")" ;
        N/=d ;

        char PC[2] ;

        mpqslog <<  endl << input_str <<" = " ;
        if( d>N ) Swap(d,N) ;                            // sort
        // primality test
        if( APRCL(d) ) { PC[0]='P' ; mpqslog << "P" ; }
        else         { PC[0]='C' ; mpqslog << "C" ; }
        mpqslog << len10(d) << " * " ;
        if( APRCL(N) ) { PC[1]='P' ; mpqslog << "P" ; }
        else           { PC[1]='C' ; mpqslog << "C" ; }
        mpqslog << len10(N)  ;

        mpqslog << endl << PC[0] << len10(d) <<" = " << d ;
        mpqslog << endl << PC[1] << len10(N) <<" = " << N ;

        mpqslog << endl << "cputime " << cputime ;
        mpqslog.close() ;
        delete [] input_str ;
        goto Input_Number ;
    }
    else {
        delete [] input_str ;
    }

    return 0 ;

}

///////////////////////////////////////////
//
//  merge duplicate data
//
//  줿 (f) ǡ ˴ؤƥȤ
//
static long MergeDuplicatedData(long nF)
{
    LONG3* fp=NULL ;
    long j,k ;
    long fxpos ;

    fp=new LONG3[nF] ;
    if( !fp ) Error(allocate error) ;
    fxpos=mpqsfx.tellp() ;
    mpqsfxix.seekp(0) ;
    for( j=0 ; j<nF ; j++ ) {
        mpqsfxix.read((char*)&(fp[j][2]),sizeof(long)) ;     // file pointer
        mpqsfx.seekp(fp[j][2]) ;
        read(mpqsfx,_fx) ;
        fp[j][1]=(unsigned long)_fx ;       // fx ΰܤ򥭡ˤ
        fp[j][0]=len(_fx) ;                         //  2 ŸǤĹ
    }
    QSortL2A(nF,12,0,4,(long*)fp) ;

    for( k=0,j=1 ; j<nF ; j++ ) {           // ʣǡΤƤ
        if( fp[j-1][1]==fp[j][1] ) {
            mpqsfx.seekp(fp[j-1][2]) ;
            read(mpqsfx,_fx) ;
            if( sign(_fx)<0 ) ~_fx ;
            mpqsfx.seekp(fp[j][2]) ;
            read(mpqsfx,_t) ;
            if( sign(_t)<0 ) ~_t ;
            if( _fx==_t ) {                      // ܤפΤΤ
                 k++ ;
                 fp[j-1][0]=0 ;                 // discard
            }
        }
    }
    if( k>0 ) {
posy++ ;
        cout << endl << k << " duplicated data merged " << flush ;
        QSortL2A(nF,12,0,8,(long*)fp) ;
    }

    QSortL2A(nF-k,12,0,8,(long*)fp+3*k) ;    // sort on length

    mpqsfxix.seekp(0) ;
    for( j=k ; j<nF ; j++ )              // ޡη̡Ȥη̤񤭽Ф
        mpqsfxix.write((char*)&fp[j][2],sizeof(long)) ;


    if( fp!=NULL ) delete [] fp ;
    mpqsfx.seekp(fxpos) ;
    return( nF-k ) ;

}

///////////////////////////////////////////
//
//  generate (F)-type data
//    from (P)-type data and (PP)-type data
//
static long LargePrime(long needF,long& nLPP)
{
    Mint fx ;
    LONG3* ppp3 ;
    long* ppp ;
    long j,k,l,l1,l2 ;
    long k0(0),k1 ;
    unsigned long p1,p2 ;
    long newFF(0) ;
    long ifp[MaxCycle+100] ;

    if( nLPP==0 ) return 0 ;
    ppp3=new LONG3[nLPP] ;
    if( !ppp3 ) Error( ppp3 allocate error) ;

    ppp=(long*)ppp3 ;
    long ppfxpos ;

    ppfxpos=mpqsppfx.tellp() ;
    long ppppos=mpqsppp.tellp() ;

    mpqsppp.seekp(0) ;
    mpqsppp.read((char*)ppp,12*nLPP) ;

         // sort on first key p2 and second key p1
    QSortL2A(nLPP-k0,12,4,0,ppp+3*k0) ;
    do {
        for( k=0,j=k0+1 ; j<nLPP ; j++ ) {    // discard duplicate data
            if( ppp3[j][0]==ppp3[j-1][0] && ppp3[j][1] ==ppp3[j-1][1] ) {

                l1=ppp3[j-1][2] ;
                if( l1<0 ) l1=-l1 ;
                else continue ;         // cannot discard because this data is already combined.

                l2=ppp3[j][2] ;
                if( l2<0 ) l2=-l2 ;
                else continue ;        // same as above

                mpqsppfx.seekp(l1) ;
                mpqsppfx.read((char*)&p1,sizeof(p1)) ;
                mpqsppfx.read((char*)&p1,sizeof(p1)) ;
                read(mpqsppfx,_fx) ;
                if( sign(_fx)<0 ) ~_fx ;

                mpqsppfx.seekp(l2) ;
                mpqsppfx.read((char*)&p1,sizeof(p1)) ;
                mpqsppfx.read((char*)&p1,sizeof(p1)) ;
                read(mpqsppfx,_t) ;
                if( sign(_t)<0 ) ~_t ;

                if( _fx==_t ) {
                    ppp3[j-1][1]=0 ;
                    k++ ;
                }
            }
        }
        if( k>0 ) {
            QSortL2A(nLPP-k0,12,4,0,ppp+3*k0) ;
            k0 += k ;
posy++ ;
        }
    } while( k!=0 ) ;
    if( k0>0 ) cout << endl << k0 << " duplicated data merged in Large Prime" ;
    k1=k0 ;

    for( l1=0 ; l1<MaxSearchLoop ; l1++ ) {
        for( j=k0 ; j<nLPP-1 ; j++ ) {
            if( ppp3[j][1]<=1 ) continue ;      // already combined
            if( ppp3[j+1][1]==ppp3[j][1] ) {    // try to comibine

                l=mpqspair.tellp() ;
                                     // indices of combined data
                mpqspair.write((char*)(ppp+3*j+2),sizeof(long)) ;
                mpqspair.write((char*)(ppp+3*(j+1)+2),sizeof(long)) ;
                ppp3[j][2]=l ;   // index to combination file
                ppp3[j][1]=ppp3[j+1][0] ;
                if( ppp3[j][0]==ppp3[j][1] ) ppp3[j][1]=1 ;
            }
        }

        QSortL2A(nLPP-k0,12,4,0,ppp+3*k0) ;

        if( ppp3[k0][1]!=1 ) break ;         // new pairing is not found any more
        else {                               // repeat algorithm
            for( j=k0 ; j<nLPP ; j++ ) {
                if( ppp3[j][1]>1 ) { k0=j ; break ; }
            }
        }

    }

    long pairpos=mpqspair.tellp() ;          // save file pointer

    int is,ie ;
    int pptype ;


    for( j=k1 ; j<k0 ; j++ ) {
        is=ie=0 ;                            // make node list empty
        if( ppp3[j][2]>0 ) {
            mpqspair.seekp(ppp3[j][2]) ;     // see combination file
            mpqspair.read((char*)&l1,sizeof(l1)) ;
            mpqspair.read((char*)&l2,sizeof(l2)) ;
            ifp[ie++]=l1 ;
            ifp[ie++]=l2 ;
        }
        else ifp[ie++]=ppp3[j][2] ; // maybe error
                                    // combination file ˤбϤʤ
                                    // ȤϤʤϤ.
                                    // ,p1=p2=1 ΤΤޤȤ
                                    // 礬
        for( k=is ; k<ie ; ) {      // envelope nodes to real nodes
            if( ifp[k]<=0 ) k++ ;   // this node is a leaf.
            else {                  // this node has children.
                mpqspair.seekp(ifp[k])  ;   // add children to node list
                mpqspair.read((char*)&l1,sizeof(l1)) ;
                mpqspair.read((char*)&l2,sizeof(l2)) ;
                ifp[k]=l1 ;         // replace parent node by its child
                ifp[ie++]=l2 ;      // add another child to node list
            }
            if( ie>MaxCycle ) break ;  //  long cycle
        }
        if( ie>MaxCycle ) continue ;  // discard long cycle
        if( --needF<0 ) break ;
        newFF++ ;
    }
    if( needF>0 ) {                 // enough data have not been gained.
        mpqsppfx.seekp(ppfxpos) ;   // reset file pointer
        mpqspair.seekp(pairpos) ;
        mpqsppp.seekp(ppppos) ;
        delete [] ppp3 ;
        return -newFF ;             // try to sieve more
    }
    else {
        newFF=0 ;
    }


    for( j=k1 ; j<k0 ; j++ ) { // generate f-type from p-type and pp-type
        is=ie=0 ;              // make node list  empty
        pptype=0 ;
        if( ppp3[j][2]>0 ) {   // combination data
            mpqspair.seekp(ppp3[j][2]) ;     // see combination file
            mpqspair.read((char*)&l1,sizeof(l1)) ;
            mpqspair.read((char*)&l2,sizeof(l2)) ;
            ifp[ie++]=l1 ;
            ifp[ie++]=l2 ;
        }
        else ifp[ie++]=ppp3[j][2] ;

        for( k=is ; k<ie ; ) {
            if( ifp[k]<=0 ) k++ ;
            else {                  // in case combination data
                mpqspair.seekp(ifp[k])  ;
                mpqspair.read((char*)&l1,sizeof(l1)) ;  // read its children
                mpqspair.read((char*)&l2,sizeof(l2)) ;
                ifp[k]=l1 ;                        // replace by child
                ifp[ie++]=l2 ;
            }
            if( ie>MaxCycle ) break ;
        }
        if( ie>MaxCycle ) continue ;
        pp=1 ;
        _fx=1 ;
        sq=1 ;                           //  (ss'..)^2*(pp'..)^-2=ff'.../(pp'..) mod pN

        for( k=0 ; k<ie ; k++ ) {
            if( ifp[k]>0 ) continue ;

            mpqsppfx.seekp(-ifp[k]) ;
            mpqsppfx.read((char*)&p1,sizeof(p1)) ;
            if( p1!=1 ) pptype++ ;
            pp*=p1 ;                                 // product of large primes
            mpqsppfx.read((char*)&p2,sizeof(p2)) ;
            pp*=p2 ;                                 // product of large primes pp'..
            read(mpqsppfx,_t) ;           // value of quadaratic polynomial f
            _fx*=_t ;                     // product of f
            if( p1!=1 ) {                 // divide by large prime
                _fx/=p1 ;
            }
//cout << endl << _fx ;
            _fx/=p2 ;
            read(mpqsppfx,_t) ;           // square part
            (sq*=_t)%=pN    ;             // ss'..
        }
        if( ie<=0 ) continue ;
        pp=mod(isqrt(pp),pN) ;
        sq*=invmodp(pp,pN) ;
        sq=mod(sq,pN) ;                  // product by inverse of square part
#ifdef MPQS_DEBUG
{
        unsigned long nPrimes ;
        unsigned long* primes ;
        ReadPrimes(nPrimes,primes)  ;
        fx=_fx ;
        if( !BitDecomp(nPrimes,primes,fx,v2) ) {
            BitDecomp(nPrimes,primes,fx,v2) ;
            cout << endl << "Bit Decomp Error in Large Prime Variation " ;
            delete [] primes ;
            Error(de) ;
            continue ;
        }
        delete [] primes ;
}
#endif


        l=mpqsfx.tellp() ;
        mpqsfxix.write((char*)&l,sizeof(l)) ;

        write(mpqsfx,_fx) ;         // _fx is fully factored data
        write(mpqsfx,sq) ;

        if( pptype<10 ) npp[pptype]=npp[pptype]+1 ;
        else npp[9]++ ;

        newFF++ ;
    }

    if( k0>0 )  {

        mpqsppp.seekp(0) ;
                               // write data which are not combined.
        mpqsppp.write((char*)(ppp+3*k0),12*(nLPP-k0)) ;

        nLPP-=k0 ;
    }

    mpqsppfx.seekp(ppfxpos) ;    // reset file pointer
    mpqspair.seekp(pairpos) ;

    delete [] ppp3 ;            // delete work triplets

    return newFF ;
}

////////////////////////////////////////
//
//  make matrix
//
void SetMatrix(::fstream& mpqsfx,unsigned long* ifx,     // (I) file pointer
               unsigned long nX,                         // (I) number of relations to be needed
               unsigned long nF,                         // (I) number of smooth data
               unsigned long nPrimes,                    // (I) size of factor base
               unsigned long* prime)                     // (I) primes of factor base
{
    unsigned long* warray=new unsigned long[nPrimes] ;   // allocate work array
    unsigned long j,k,nbit,ir ;
    Mint fx ;
    TBit V ;
    unsigned long* v=new unsigned long[(nPrimes+31)/32] ; // bit vector

    V.SetBitString(nPrimes,v) ;

    elem.open(elemfile,ios::binary|ios::out) ;
    elem.seekp(0) ;
    for( ir=j=0 ; j<nF && ir<nX ; j++ ) {
        mpqsfx.seekp(ifx[j]) ;
        read(mpqsfx,fx) ;

        if( !BitDecomp(nPrimes,prime,fx,v) ) {   // decompose by the factor base
#if 1
            Error(decomp error in SetMatix) ;
#else
            mpqsfx.seekp(ifx[j]) ;
            read(mpqsfx,fx) ;
            mpqsfx.seekp(ifx[j]) ;
            read(mpqsfx,fx) ;
            continue ;
#endif
        }
        for( nbit=0,k=0 ; k<nPrimes ; k++ ) {
            if( V[k] ) warray[nbit++]=k ;       // set indices with non-zero bit
        }
        ir++ ;
                                                // write indices of non-zero entries
        elem.write((char*)&nbit,sizeof(unsigned long)) ;   // number of non-zero bits
        if( nbit!=0 ) elem.write((char*)warray,nbit*sizeof(unsigned long)) ; // write elements
    }

    elem.close() ;

    delete [] warray ;
    delete [] v ;
}


