//
//	Copyright (C) by S.Tomabechi 1999
//	All rights reserved.
//
//	block vector class
//	intended to use for Block Lanczos method
//

#ifdef __BORLANDC__
#pragma inline
#endif
#include <mem.h>

#include "bvector.hpp"
#include "TBit.hpp"
#include "SMatrix.hpp"
#include "Error.hpp"


///////////////////////////
//
// constructor
//
block_vector::block_vector(const unsigned long _ndim):ndim(0),v(NULL)
{
	ndim=_ndim ;
	v=new unsigned long [ndim] ;
}

//////////////////////////
//
// copy constructor
//
block_vector::block_vector(const block_vector& bv)
{
	ndim=bv.ndim ;
	v=new unsigned long[ndim] ;
	memcpy(v,bv.v,ndim*sizeof(unsigned long)) ;
}

///////////////////////////
//
// assign operator
//
block_vector& block_vector::operator=(const block_vector& bv)
{
	if( this==&bv ) return *this ;				// same

	if( ndim==bv.ndim ) ;								// same dimension
	else {
		if( v ) delete [] v ;								// realloc
		ndim=bv.ndim ;
		v=new unsigned long[ndim] ;
	}
	memcpy(v,bv.v,ndim*sizeof(unsigned long)) ;
	return *this ;

}

///////////////////////////
//
// destructor
//
block_vector::~block_vector()
{
	if( v ) delete [] v ;
}


::ostream& operator<<(::ostream& os,const block_vector& bvec)
{
	TBit B ;
	for( unsigned long j=0 ; j<bvec.ndim ; j++ ) {
		B.SetBitString(WordSize,bvec.v+j) ;
		os << endl << B ;
	}
	return os ;
}


//
// production of sparse matrix and block vector
//
/*********************
block_vector operator*(const SparseMatrix& m,const block_vector& w)
{
	block_vector sv(m.nrow) ;
	unsigned long b ;
	unsigned long j,k ;

	for( j=0 ; j<m.nrow ; j++ ) {
		b=0 ;
		for( k=0 ; k<m.w_row[j] ; k++ ) {
			b^=w.v[m.mat[j][k]] ;
		}
		sv.SetRow(j,b) ;
	}
	return sv ;
}
******************/

//////////////////////////////////////////////
//
// product of sparse matrix and block vector
//
//	*this=m*w
//
block_vector& block_vector::isMul(const SparseMatrix& m,const block_vector& w)
{
	bool same=(this==&w) ;
	unsigned long* wv ;
//	unsigned long b ;
	unsigned long j,k ;
	if( same ) {
		ndim=m.nrow ;
		wv=new unsigned long[ndim] ;
		unsigned long* temp ;
		temp=v ;
		v=wv ;
		wv=temp ;
	}
	else {
		if( ndim!=m.nrow ) {
			delete [] v ;
			ndim=m.nrow ;
			v=new unsigned long[ndim] ;
		}
		wv=w.v ;
	}

	__asm {
		mov esi,wv

		mov edi,this
#ifdef __BORLANDC__
		mov edi,dword ptr [edi].(class block_vector)v
#else
		mov edi,dword ptr [edi][BV_VECTOR]
#endif

		mov ebx,m
#ifdef __BORLANDC__
		mov ecx,dword ptr [ebx].(SparseMatrix)nrow
#else
		mov ecx,dword ptr [ebx][SM_NROW]
#endif
		dec ecx
		mov j,ecx

	}
OUT_LOOP:
	__asm {
//		dec j
//		js	RET1

		mov ebx,m
		mov edx,j

#ifdef __BORLANDC__
		mov ecx,dword ptr [ebx].(class SparseMatrix)w_row
#else
		mov ecx,dword ptr [ebx][SM_WROW]
#endif
		mov ecx,dword ptr [ecx][4*edx]										// ecx <-	 w_row[j]

#ifdef __BORLANDC__
		mov eax,dword ptr [ebx].(class SparseMatrix)mat
#else
		mov eax,dword ptr [ebx][SM_MAT]
#endif
		mov ebx,dword ptr [eax][4*edx]									 // mat[j]


		xor edx,edx
		dec ecx								// w_row[j]-1
	}
INNER_LOOP:
	__asm {

		mov eax,dword ptr [ebx][4*ecx]		 // m.mat[j][k]
		mov eax,dword ptr [esi][4*eax]		 // wv[m.mat[j][k]]

		xor edx,eax

		dec ecx
		jns	INNER_LOOP
//		jmp INNER_LOOP
	}
CONT:
SET_RESULT:
	__asm {
		mov ecx,j
//		mov edx,b
		mov dword ptr [edi][4*ecx],edx
		dec j
		jns OUT_LOOP
//		jmp OUT_LOOP

	}
RET1:

/********

	for( j=0 ; j<m.nrow ; j++ ) {
		b=0 ;
		for( k=0 ; k<m.w_row[j] ; k++ ) {
			b^=wv[m.mat[j][k]] ;
		}
		v[j]=b ;
	}
*********/
	if( same ) delete [] wv ;
	return *this ;

}

///////////////////////////////////////
//
// product of block vectors as matrix
//
block_vector block_vector::operator*(const block_vector& a)
{

/************
	block_vector ta(ndim) ;
	unsigned long b ;
	unsigned long j,k,j32 ;
	TBit B ;

	for( j=0 ; j<ndim ; j++ ) {
		b=0 ;
		B.SetBitString(WordSize,v+j) ;
		for( k=0 ; k<WordSize ; k++ ) {
			if( B[k] ) {
				b^=a.v[k] ;
			}
		}
		ta.v[j]=b ;
	}
	return ta ;
*************/
	block_vector tb(ndim) ;
	return tb.isMul(*this,a) ;
//	if( ta!=tb ) Error(not same result) ;
}


//////////////////////////////////////////////
//
//	product of block vectors as matrix
//
//	assumed a.ncol=b.nrow
//
//	 this=&a ł悢
//
block_vector& block_vector::isMul(const block_vector& a,const block_vector& b)
{
//	unsigned long av ;
	unsigned long j,k ;

#ifndef __SNFS__	   // NFS Ŏgꍇ͂̂悤ȂƂ͂ȂƉł
	if( this==&b ) return isMul(a,block_vector(b)) ;
#endif	

	__asm {
		mov esi,b
#ifdef __BORLANDC__
		mov esi,dword ptr [esi].(class block_vector)v
#else
		mov esi,dword ptr [esi][BV_VECTOR]
#endif

		mov edi,this
#ifdef __BORLANDC__
		mov eax,dword ptr [edi].(class block_vector)ndim
#else
		mov eax,dword ptr [edi][BV_NDIM]
#endif
		dec eax
		mov j,eax
#ifdef __BORLANDC__
		mov edi,dword ptr [edi].(class block_vector)v
#else
		mov edi,dword ptr [edi][BV_VECTOR]
#endif

		mov ebx,a
#ifdef __BORLANDC__
		mov ebx,dword ptr [ebx].(class block_vector)v
#else
		mov ebx,dword ptr [ebx][BV_VECTOR]
#endif
	}
OUT_LOOP:
	__asm {
//		dec j
//		js	RET1
		mov ecx,j
		mov eax,dword ptr [ebx][4*ecx]	 // a.v[j]
//		mov av,eax

		xor edx,edx		// c<-0

		mov ecx,WordSize
//		dec ecx
	}
INNER_LOOP:
	__asm {
		dec ecx
		js	SET_RESULT
#ifdef __SNFS__
		bt	eax,ecx
		jnc INNER_LOOP_1								// zero

		xor edx,dword ptr [esi][4*ecx]
//		jmp INNER_LOOP
	}
INNER_LOOP_1:
	__asm {
		dec ecx
#endif
		bt	eax,ecx
		jnc INNER_LOOP								// zero

		xor edx,dword ptr [esi][4*ecx]
		jmp INNER_LOOP
	}
SET_RESULT:
	__asm {
		mov ecx,j
		mov dword ptr [edi][4*ecx],edx
		dec j
		jns OUT_LOOP
//		jmp OUT_LOOP

	}
RET1:
	return *this ;

/**************
	for( j=0 ; j<ndim ; j++ ) {
		av=a.v[j] ;
		c=0 ;
		for( k=0 ; k<WordSize ; k++ ) {
			__asm {
				mov ecx,k
				bt	av,ecx
				jnc CONT
			}
			c^=b.v[k] ;
CONT:
		}
		v[j]=c ;
	}
	return *this ;
***********/

}





//
//	 calculate m^t*w
//
block_vector& block_vector::isMulT(const SparseMatrix& m,const block_vector& w)
{
//	block_vector sv(m.ncol) ;
	unsigned long b ;
	unsigned long j,k ;

	if( ndim!=m.ncol ) {
		if( v ) delete [] v ;
		ndim=m.ncol ;
		v=new unsigned long[ndim] ;
	}


	__asm {
											// initialize *this
		mov edi,this
#ifdef __BORLANDC__
		mov ecx,dword ptr [edi].(class block_vector)ndim
		mov edi,dword ptr [edi].(class block_vector)v
#else
		mov ecx,dword ptr [edi][BV_NDIM]
		mov edi,dword ptr [edi][BV_VECTOR]
#endif
		mov esi,edi
		xor eax,eax
		rep stosd														// clear this->v

		mov edi,esi

		mov esi,w
#ifdef __BORLANDC__
		mov esi,dword ptr [esi].(class block_vector)v
#else
		mov esi,dword ptr [esi][BV_VECTOR]
#endif


		mov ebx,m

#ifdef __BORLANDC__
		mov ecx,dword ptr [ebx].(SparseMatrix)nrow
#else
		mov ecx,dword ptr [ebx][SM_NROW]
#endif
		dec ecx
		mov j,ecx

	}
OUT_LOOP:
	__asm {
//		dec j
//		js	RET1


//		mov ecx,j
//		mov eax,dword ptr [ebx][4*ecx]	 // a.v[j]
//		mov av,eax


		mov ebx,m
		mov edx,j

#ifdef __BORLANDC__
		mov ecx,dword ptr [ebx].(class SparseMatrix)w_row
#else
		mov ecx,dword ptr [ebx][SM_WROW]
#endif
		mov ecx,dword ptr [ecx][4*edx]										// ecx <-	 w_row[j]

#ifdef __BORLANDC__
		mov eax,dword ptr [ebx].(class SparseMatrix)mat
#else
		mov eax,dword ptr [ebx][SM_MAT]
#endif
		mov ebx,dword ptr [eax][4*edx]									 // mat[j]

		mov edx,dword ptr [esi][4*edx]									 // w.v[j]


//		xor edx,edx
		dec ecx
	}
INNER_LOOP:
	__asm {

#ifdef __SNFS__
		mov eax,dword ptr [ebx][4*ecx]		 // m.mat[j][k]
		xor dword ptr [edi][4*eax],edx		 //	v[m.mat[j][k]]^=w.v[j] ;
		dec ecx		
		js  SET_RESULT												 // ecx=k
#endif
		mov eax,dword ptr [ebx][4*ecx]		 // m.mat[j][k]
		xor dword ptr [edi][4*eax],edx		 //	v[m.mat[j][k]]^=w.v[j] ;
		dec ecx														 // ecx=k
		jns	INNER_LOOP
//		jmp
	}
SET_RESULT:
	__asm {
		dec j
		jns OUT_LOOP
	}
RET1:
/***********
	for( j=0 ; j<m.nrow ; j++ ) {
		for( k=0 ; k<m.w_row[j] ; k++ ) {
			v[m.mat[j][k]]^=w.v[j] ;
		}
	}
***********/
	return *this ;


}



block_vector block_vector::operator+(const block_vector& b)
{
	block_vector s(ndim) ;
	return s.isAdd(*this,b) ;

}


block_vector& block_vector::isAdd(const block_vector& a,const block_vector& b)
{
	// dimension Ă邩ǂ̃`FbN̓T{

	__asm {
		mov edi,this
		mov esi,a
		mov ebx,b
#ifdef __BORLANDC__
		mov ecx,dword ptr [esi].(class block_vector)ndim
		mov esi,dword ptr [esi].(class block_vector)v
		mov edi,dword ptr [edi].(class block_vector)v
		mov ebx,dword ptr [ebx].(class block_vector)v
#else
		mov ecx,dword ptr [esi][BV_NDIM]
		mov esi,dword ptr [esi][BV_VECTOR]
		mov edi,dword ptr [edi][BV_VECTOR]
		mov ebx,dword ptr [ebx][BV_VECTOR]
#endif
		dec ecx
	}
LOOP1:
	__asm {
#ifdef __SNFS__
		mov edx,dword ptr [esi][4*ecx]
		xor edx,dword ptr [ebx][4*ecx]
		mov dword ptr [edi][4*ecx],edx
		dec ecx
		js RET1 
#endif		
		mov edx,dword ptr [esi][4*ecx]
		xor edx,dword ptr [ebx][4*ecx]
		mov dword ptr [edi][4*ecx],edx
		dec ecx
		jns LOOP1
	}
RET1:
	return *this ;

//	unsigned long j ;
//	for( j=0 ; j<ndim ; j++ ) {
//		v[j]=a.v[j]^b.v[j] ;
//	}
//	return *this ;
}





block_vector& block_vector::isAND(const block_vector& a,const unsigned long c)
{
	// dimension Ă邩ǂ̃`FbN̓T{

	__asm {
		mov edi,this
		mov esi,a
		mov ebx,c
#ifdef __BORLANDC__
		mov ecx,dword ptr [esi].(class block_vector)ndim
		mov esi,dword ptr [esi].(class block_vector)v
		mov edi,dword ptr [edi].(class block_vector)v
#else
		mov ecx,dword ptr [esi][BV_NDIM]
		mov esi,dword ptr [esi][BV_VECTOR]
		mov edi,dword ptr [edi][BV_VECTOR]
#endif
		dec ecx
	}
LOOP:
	__asm {
#ifdef __SNFS__
		mov edx,dword ptr [esi][4*ecx]
		and edx,ebx
		mov dword ptr [edi][4*ecx],edx
		dec ecx
#endif		
		mov edx,dword ptr [esi][4*ecx]
		and edx,ebx
		mov dword ptr [edi][4*ecx],edx
		dec ecx
		jns LOOP
	}
RET1:
	return *this ;

//	unsigned long j ;
//	for( j=0 ; j<ndim ; j++ ) {
//		v[j]=a.v[j]&c ;
//	}
//	return *this ;
}

bool block_vector::isZero(void)
{
	__asm {
		mov ebx,this
#ifdef __BORLANDC__
		mov ecx,dword ptr [ebx].(class block_vector)ndim
		mov ebx,dword ptr [ebx].(class block_vector)v
#else
		mov ecx,dword ptr [ebx][BV_NDIM]
		mov ebx,dword ptr [ebx][BV_VECTOR]
#endif
		xor eax,eax													// eax <- 0
		dec ecx
	}
LOOP1:
	__asm {
		cmp dword ptr[ebx][4*ecx],eax				// zero ?
		jne NOT_ZERO
		dec ecx
		jns LOOP1
	}
ZERO:
	return true ;
NOT_ZERO:
	return false ;

}


///////////////////////////////////////
//
// inner product of block vectors
//
block_vector& block_vector::isIpro(const block_vector& a,const block_vector& b)
{
	unsigned long adim,j ;
//	unsigned long av ;
#ifndef __SNFS__                // nfs ł͂̃`FbNKvȂ̂ŊO
	if( ndim!=WordSize ) {
		delete [] v ;
		ndim=WordSize ;
		v=new unsigned long[WordSize] ;
	}
#endif 

	__asm {

		mov esi,b

#ifdef __BORLANDC__
		mov esi,dword ptr [esi].(class block_vector)v

		mov edi,this
		mov edi,dword ptr [edi].(class block_vector)v

		mov ebx,a
		mov eax,dword ptr [ebx].(class block_vector)ndim
		mov adim,eax
		mov ebx,dword ptr [ebx].(class block_vector)v
#else
		mov esi,dword ptr [esi][BV_VECTOR}

		mov edi,this
		mov edi,dword ptr [edi][BV_VECTOR]

		mov ebx,a
		mov eax,dword ptr [ebx][BV_NDIM]
		mov adim,eax
		mov ebx,dword ptr [ebx][BV_VECTOR]
#endif

		mov eax,WordSize
		dec eax

	}
OUT_LOOP:
	__asm {
//		dec eax
//		js	RET1

		xor edx,edx										 // c<-0

		mov ecx,adim   // adim ͋Ƃ͌Ȃ !!
//		dec ecx
	}
INNER_LOOP:
	__asm {
		dec ecx
		js	SET_RESULT
#ifdef __SNFS__
		bt dword ptr [ebx][4*ecx],eax
		jnc INNER_LOOP_1

		xor edx,dword ptr [esi][4*ecx]	 // c^=b.v[k]
	}
INNER_LOOP_1:			
	__asm {	
		dec ecx
		js SET_RESULT
#endif
		bt dword ptr [ebx][4*ecx],eax
		jnc INNER_LOOP

		xor edx,dword ptr [esi][4*ecx]	 // c^=b.v[k]
	
		jmp INNER_LOOP
	}
SET_RESULT:
	__asm {
		mov dword ptr [edi][4*eax],edx	// v[k]	 <- c
		dec eax
		jns OUT_LOOP
//		jmp OUT_LOOP

	}
RET1:
	return *this ;
}
/*********
block_vector Ipro(const block_vector& a,const block_vector& b)
{
	block_vector ab(32) ;
	unsigned long r ;
	unsigned long j,k ;
	TBit B ;
	B.SetBitString(WordSize*a.ndim,a.v) ;

	for( j=0 ; j<WordSize ; j++ ) {
		r=0 ;
		for( k=0 ; k<a.ndim ; k++ ) {
			if( B[k*WordSize+j] ) {
				r^=b.v[k] ;
			}
		}
		ab.v[j]=r ;
	}

	return ab ;
}

**********/
unsigned long block_vector::operator*(const unsigned long pv)
{
	unsigned long j,k,l ;
	unsigned long w ;
	unsigned long p(0) ;
	TBit wB,pB ;
	wB.SetBitString(WordSize,&w) ;
	pB.SetBitString(WordSize,&p) ;
	for( j=0 ; j<ndim ; j++ ) {
		w=v[j]&pv ;
		for( l=k=0 ; k<WordSize ; k++ ) {
			if( wB[k] ) l++ ;
		}
		if( l&1 ) pB.BitOn(j) ;
		else			pB.BitOff(j) ;
	}

	return p ;

}

////////////////////////////
//
//	comparison operator
//
bool block_vector::operator==(const block_vector& b)
{
	unsigned long j ;
	if( ndim!=b.ndim ) return false ;
	for( j=0 ; j<ndim ; j++ ) {
		if( v[j]!=b.v[j] ) return false ;
	}
	return true ;
}



///////////////////////////////
//
// swap block vectors
//
void Swap(block_vector& a,block_vector& b)
{
	__asm {
		mov esi,a
		mov edi,b
#ifdef __BORLANDC__
		mov	 eax,dword ptr [esi].(class block_vector)ndim
		xchg eax,dword ptr [edi].(class block_vector)ndim
		mov	 dword ptr [esi].(class block_vector)ndim,eax

		mov	 eax,dword ptr [esi].(class block_vector)v
		xchg eax,dword ptr [edi].(class block_vector)v
		mov	 dword ptr [esi].(class block_vector)v,eax
#else

		mov	 eax,dword ptr [esi][BV_NDIM]
		xchg eax,dword ptr [edi]
		mov	 dword ptr [esi],eax

		mov	 eax,dword ptr [esi][BV_VECTOR]
		xchg eax,dword ptr [edi][BV_VECTOR]
		mov	 dword ptr [esi][BV_VECTOR],eax
#endif
	}
}
