//
//  Copyright (C) by S.Tomabechi 1994
//  All rights reserved.
//
//  Elliptic integral
//

#include "Fint.hpp"
#include "Error.hpp"

//
// elliptic integral of 1-st kind
//
Fint K(const Fint& a,const Fint& b)
{
  if( a*b==0 ) Error( invalid parameter in K) ;
  return pi(a.fdigit())/agm(a,b)/2 ;
}

//
//  complete elliptic integral of 1-st kind
//
Fint K(const Fint& k)
{
  return K(1,sqrt(1-k*k)) ;
}

//
//  elliptic integral of 2-nd kind
//
Fint E(Fint a,Fint b)
{
  if( a*b==0 ) Error( invalid parameter in E) ;

//  Fint k=K(a,b) ;
  Fint t ;
  Fint s(a*a) ;
  Fint c ;
  long ishift=-1 ;
  t=(a+b)/2 ;
  s -= 2*t*(a-t) ;
  do {
    t=(a+b)/2 ;
    c=a-t ;
    b=sqrt(a*b) ;
    a=t ;
    s -= ((c*c)<<(++ishift)) ;
  } while( a>b ) ;
  return s*pi(a.fdigit())/a/2 ;
}


//
//  complex elliptic integral of 2-nd kind
//
Fint E(const Fint& k)
{
  return E(1,sqrt(1-k*k)) ;
}

//
// (incomplete) elliptic integral of 1-st kind
// phi,theta
//
// | [O  Ț_  gS ɌvZ@ڂĂ
// ʂǂ悭Ȃ
//
Fint F(Fint p,const Fint& t)
{
  Fint a(1),b(cos(t)) ;
  if( b==0 ) return log((1+sin(p))/cos(p)) ;
  Fint a1 ;
  Fint half(2,1) ;
  (half=1) /= 2 ;
  long n=0 ;
  do {
    p += floor(p/pi()+half)*pi()+atan(b/a*tan(p)) ;
    a1=(a+b)/2 ;
    b=sqrt(a*b) ;
    a=a1 ;
    n++ ;
  } while( a>b ) ;
  b<<=n ;
  return p/b ;
}


//
//  (incomplete) elliptic integral of 2-nd kind
//
Fint E2(Fint p,const Fint& t)
{

Fint pp(p) ;
  if( t==pi()/2 ) return sin(p) ;
  if( t==0 ) return p ;

  Fint a(1),b(cos(t)) ;
  Fint a1 ;
  Fint half(2,1) ;
  (half=1) /= 2 ;

  Fint c ;
  Fint cs(0) ;
  Fint c2(a*a) ;
  long ishift=-1 ;
  long n=0 ;

  a1=(a+b)/2 ;
  c2 -= 2*a1*(a-a1) ;
  do {
    p += floor(p/pi()+half)*pi()+atan(b/a*tan(p)) ;
    a1=(a+b)/2 ;
    c=a-a1 ;
    cs += c*sin(p) ;
    b=sqrt(a*b) ;
    a=a1 ;
    c2 -= ((c*c)<<(++ishift)) ;
    n++ ;
  } while( a>b) ;
//cout << endl << (E(sin(t))/K(sin(t)))*F(pp,t)+cs ;
  b<<=n ;
  return p/b*c2+cs ;

}
